# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Plotting routines for Brian

Functions:

* ``raster_plot(monitors...,options...)``
* ``hist_plot(monitor,options...)``
"""

__docformat__ = "restructuredtext en"

__all__ = ['plot', 'show', 'figure', 'xlabel', 'ylabel', 'title', 'axis',
           'raster_plot', 'raster_plot_spiketimes', 'hist_plot']

try:
    from pylab import plot, show, figure, xlabel, ylabel, title, axis, xlim
    import pylab, matplotlib
except:
    plot, show, figure, xlabel, ylabel, title, axis, xlim = (None,)*8
from stdunits import *
import magic
from connections import *
from monitor import *
from monitor import HistogramMonitorBase
from network import network_operation
from clock import EventClock
import warnings
from log import *
from numpy import amax, amin, array, hstack
import bisect

def _take_options(myopts, givenopts):
    """Takes options from one dict into another
    
    Any key defined in myopts and givenopts will be removed
    from givenopts and placed in myopts.
    """
    for k in myopts.keys():
        if k in givenopts:
            myopts[k] = givenopts.pop(k)


def raster_plot(*monitors, **plotoptions):
    """Raster plot of a :class:`SpikeMonitor`
    
    **Usage**
    
    ``raster_plot(monitor,options...)``
        Plots the spike times of the monitor
        on the x-axis, and the neuron number on the y-axis
    ``raster_plot(monitor0,monitor1,...,options...)``
        Plots the spike times
        for all the monitors given, with y-axis defined by placing a spike
        from neuron n of m in monitor i at position i+n/m
    ``raster_plot(options...)``
        Guesses the monitors to plot automagically
    
    **Options**
    
    Any of PyLab options for the ``plot`` command can be given, as well as:
    
    ``showplot=False``
        set to ``True`` to run pylab's ``show()`` function
    ``newfigure=False``
        set to ``True`` to create a new figure with pylab's ``figure()`` function
    ``xlabel``
        label for the x-axis
    ``ylabel``
        label for the y-axis
    ``title``
        title for the plot    
    ``showgrouplines=False``
        set to ``True`` to show a line between each monitor
    ``grouplinecol``
        colour for group lines
    ``spacebetweengroups``
        value between 0 and 1 to insert a space between
        each group on the y-axis
    ``refresh``
        Specify how often (in simulation time) you would like the plot to
        refresh. Note that this will only work if pylab is in interactive mode,
        to ensure this call the pylab ``ion()`` command.
    ``showlast``
        If you are using the ``refresh`` option above, plots are much quicker
        if you specify a fixed time window to display (e.g. the last 100ms).
    ``redraw``
        If you are using more than one realtime monitor, only one of them needs
        to issue a redraw command, therefore set this to ``False`` for all but
        one of them.

    Note that with some IDEs, interactive plotting will not work with the
    default matplotlib backend, try doing something like this at the
    beginning of your script (before importing brian)::
    
        import matplotlib
        matplotlib.use('WXAgg')
        
    You may need to experiment, try WXAgg, GTKAgg, QTAgg, TkAgg.
    """
    if len(monitors) == 0:
        (monitors, monitornames) = magic.find_instances(SpikeMonitor)
    if len(monitors):
        # OPTIONS
        # Defaults
        myopts = {"title":"", "xlabel":"Time (ms)", "showplot":False, "showgrouplines":False, \
                  "spacebetweengroups":0.0, "grouplinecol":"k", 'newfigure':False,
                  'refresh':None, 'showlast':None, 'redraw':True}
        if len(monitors) == 1:
            myopts["ylabel"] = 'Neuron number'
        else:
            myopts["ylabel"] = 'Group number'
        # User options
        _take_options(myopts, plotoptions)
        # PLOTTING ROUTINE
        spacebetween = myopts['spacebetweengroups']
        class SecondTupleArray(object):
            def __init__(self, obj):
                self.obj = obj
            def __getitem__(self, i):
                return float(self.obj[i][1])
            def __len__(self):
                return len(self.obj)
        def get_plot_coords(tmin=None, tmax=None):
            allsn = []
            allst = []
            for i, m in enumerate(monitors):
                mspikes = m.spikes
                if tmin is not None and tmax is not None:
                    x = SecondTupleArray(mspikes)
                    imin = bisect.bisect_left(x, tmin)
                    imax = bisect.bisect_right(x, tmax)
                    mspikes = mspikes[imin:imax]
                if len(mspikes):
                    sn, st = array(mspikes).T
                else:
                    sn, st = array([]), array([])
                st /= ms
                if len(monitors) == 1:
                    allsn = [sn]
                else:
                    allsn.append(i + ((1. - spacebetween) / float(len(m.source))) * sn)
                allst.append(st)
            sn = hstack(allsn)
            st = hstack(allst)
            if len(monitors) == 1:
                nmax = len(monitors[0].source)
            else:
                nmax = len(monitors)
            return st, sn, nmax
        st, sn, nmax = get_plot_coords()
        if myopts['newfigure']:
            pylab.figure()
        if myopts['refresh'] is None:
            line, = pylab.plot(st, sn, '.', **plotoptions)
        else:
            line, = pylab.plot([], [], '.', **plotoptions)
        if myopts['refresh'] is not None:
            pylab.axis(ymin=0, ymax=nmax)
            if myopts['showlast'] is not None:
                pylab.axis(xmin= -myopts['showlast'] / ms, xmax=0)
        ax = pylab.gca()
        if myopts['showgrouplines']:
            for i in range(len(monitors)):
                pylab.axhline(i, color=myopts['grouplinecol'])
                pylab.axhline(i + (1 - spacebetween), color=myopts['grouplinecol'])
        pylab.ylabel(myopts['ylabel'])
        pylab.xlabel(myopts['xlabel'])
        pylab.title(myopts["title"])
        if myopts["showplot"]:
            pylab.show()
        if myopts['refresh'] is not None:
            @network_operation(clock=EventClock(dt=myopts['refresh']))
            def refresh_raster_plot(clk):
                if matplotlib.is_interactive():
                    if myopts['showlast'] is None:
                        st, sn, nmax = get_plot_coords()
                        line.set_xdata(st)
                        line.set_ydata(sn)
                        ax.set_xlim(0, amax(st))
                    else:
                        st, sn, nmax = get_plot_coords(clk._t - float(myopts['showlast']), clk._t)
                        ax.set_xlim((clk.t - myopts['showlast']) / ms, clk.t / ms)
                        line.set_xdata(array(st))
                        line.set_ydata(sn)
                    if myopts['redraw']:
                        pylab.draw()
                        pylab.get_current_fig_manager().canvas.flush_events()
            monitors[0].contained_objects.append(refresh_raster_plot)


def raster_plot_spiketimes(spiketimes):
    """
    Raster plot of a list of spike times
    """
    m = Monitor()
    m.source = []
    m.spikes = spiketimes
    raster_plot(m)
    t = array(spiketimes)[:,1]


def hist_plot(histmon=None, **plotoptions):
    """Plot a histogram
    
    **Usage**
    
    ``hist_plot(histmon,options...)``
        Plot the given histogram monitor
    ``hist_plot(options...)``
        Guesses which histogram monitor to use
    
    with argument:
    
    ``histmon``
        is a monitor of histogram type
    
    **Notes**
    
    Plots only the first n-1 of n bars in the histogram, because
    the nth bar is for the interval (-,infinity).
    
    **Options**
    
    Any of PyLab options for bar can be given, as well as:
    
    ``showplot=False``
        set to ``True`` to run pylab's ``show()`` function
    ``newfigure=True``
        set to ``False`` not to create a new figure with pylab's ``figure()`` function
    ``xlabel``
        label for the x-axis
    ``ylabel``
        label for the y-axis
    ``title``
        title for the plot
    """
    if histmon is None:
        (histmons, histmonnames) = magic.find_instances(HistogramMonitorBase)
        if len(histmons) == 0:
            raise TypeError, "No histogram monitors found."
        elif len(histmons) > 1:
            log_info('brian.hist_plot', "Found more than one histogram monitor, using first one.")
        histmon = histmons[0]
    # OPTIONS
    # Defaults
    myopts = {"title":"", "xlabel":"Time (ms)", "ylabel":"Count", "showplot":False, 'newfigure':True }
    # User options
    _take_options(myopts, plotoptions)
    # PLOTTING ROUTINE
    if myopts['newfigure']:
        pylab.figure()
    pylab.bar(histmon.bins[:-1] / ms, histmon.count[:-1], (histmon.bins[1:] - histmon.bins[:-1]) / ms, **plotoptions)
    pylab.ylabel(myopts['ylabel'])
    pylab.xlabel(myopts['xlabel'])
    pylab.title(myopts["title"])
    if myopts["showplot"]:
        pylab.show()



'''
This program duplicates the results of Masquelier et al. 2008 - STDP finds the
start of repeating patterns in continuous spike trains.

1000 neurons are connected to a single neuron by STDP synapses. Every 200ms,
the pattern RANDOM 100ms, PATTERN 50ms RANDOM 50ms is repeated, where RANDOM
is Poisson noise, and PATTERN is the same pattern repeated. More exactly,
half the neurons are patterned, and half are random all the time.

The STDP model is standard exponential STDP, not the one from that paper, and so
the results are slightly different here.
'''
from brian import *
from brian.utils.progressreporting import ProgressReporter
import time
import pylab

class TimedPoissonGroup(PoissonGroup):
    '''
    Fires Poisson spikes during a certain interval
    
    See documentation for :class:`PoissonGroup`, with the additional feature that
    spikes will only be fired between ``start`` and ``start+duration``. If ``period``
    is specified, it will repeat with that period.
    '''
    def __init__(self, N=1, rates=0*Hz, start=0*second, duration=1*second,
                 clock=None, period=None):
        end = float(start+duration)
        start = float(start)
        if period is not None:
            period = float(period)
        def f_rates(t):
            t = float(t)
            if period is not None:
                t = t%period
            if start<t<end:
                return rates
            else:
                return 0*Hz
        PoissonGroup.__init__(self, N, rates=f_rates, clock=clock)
        
def make_poisson_spiketrain(N, rates, start, duration, **kwds):
    '''
    Returns a spike train for N Poisson neurons firing at rates between start and duration
    '''
    c = Clock(**kwds)
    G = TimedPoissonGroup(N, rates, start, duration, c)
    M = SpikeMonitor(G)
    net = Network(G, M)
    net.run(start+duration)
    return M.spikes

# appears to be just about good enough, it's more stable if you make it smaller
defaultclock.dt = 0.25*ms

initial_segment_duration = 100*ms
pattern_segment_duration = 50*ms
end_segment_duration = 50*ms
N_unpatterned = 500 # Number of neurons that are occasionally patterned
N_patterned = 500   # Noise neurons
poprate = 50*Hz
repeats = 500

# The sequence repeats many times, we plot plotseq_n plots starting from
# the repeats in plotseq_startset, to show that the repeated part of the
# pattern looks different
plotseq_n = 5
plotseq_startset = [0, int(repeats/3), int((2*repeats)/3), repeats-plotseq_n]

# Parameters for STDP model
taum = 20*ms
tau_post = 20*ms
tau_pre = 20*ms
Ee = 0*mV
Vt = -54*mV
Vr = -60*mV
El = -70*mV
taue = 5*ms
gmax = 0.004           # Maximum synaptic strength
dA_pre = .005          # Potentiation rate
dA_post = -dA_pre*1.1  # Depression rate

total_duration = initial_segment_duration + pattern_segment_duration + end_segment_duration
N = N_unpatterned + N_patterned

# This is the spike train that will be repeated
pat_spikes = make_poisson_spiketrain(
                N_patterned, poprate, initial_segment_duration,
                pattern_segment_duration, dt=defaultclock.dt)

eqs_post = '''
dV/dt = (ge*(Ee-V)+El-V)/taum  : volt
dge/dt = -ge/taue              : 1
excitatory = ge
'''

# Poisson spikes for the patterned neurons for the initial segment (PG_start),
# for the end segment (PG_end), and for the unpatterned neurons throughout
# (PG_all), and the pattern for the patterned neurons in the middle segment
# (patgroup)
PG_start = TimedPoissonGroup(
            N_patterned, poprate, 0*ms,
            initial_segment_duration)
PG_end = TimedPoissonGroup(
            N_patterned, poprate,
            initial_segment_duration+pattern_segment_duration,
            end_segment_duration)
PG_all = TimedPoissonGroup(
            N_unpatterned, poprate, 0*ms,
            initial_segment_duration+pattern_segment_duration+end_segment_duration)
patgroup = SpikeGeneratorGroup(N_patterned, pat_spikes)

G_pre = NeuronGroup(N, 'V:1', threshold=0.5, reset=0.0)
G_unpatterned = G_pre.subgroup(N_unpatterned)
G_patterned = G_pre.subgroup(N_patterned)

C_start = IdentityConnection(PG_start, G_patterned)
C_end = IdentityConnection(PG_end, G_patterned)
C_all = IdentityConnection(PG_all, G_unpatterned)
C_pat = IdentityConnection(patgroup, G_patterned)

G_post = NeuronGroup(1, eqs_post, threshold=Vt, reset=Vr)

synapses = Connection(G_pre, G_post, 'excitatory', structure='dense')
synapses.connect(G_pre, G_post, rand(len(G_pre), len(G_post))*gmax)

stdp = ExponentialSTDP(synapses, tau_pre, tau_post, dA_pre, dA_post, wmax=gmax)

M_pre = SpikeMonitor(G_pre)
M_post = SpikeMonitor(G_post)
MV_post = StateMonitor(G_post, 'V', record=True)

G_post.V = Vr

net = MagicNetwork()

weights_before = array(synapses.W).copy()
t_start = time.time()

reporter = ProgressReporter('stderr')
for i in range(repeats):
    reporter.equal_subtask(i, repeats)
    reinit_default_clock()
    # Reinitialise these by hand because we don't want to reinitialise the
    # spike and state monitors
    PG_start.reinit()
    PG_end.reinit()
    PG_all.reinit()
    G_pre.reinit()
    G_post.reinit()
    patgroup.reinit()
    net.run(total_duration, report=reporter)

t_end = time.time()
weights_after = array(synapses.W).copy()

print 'Time taken:', t_end-t_start
print 'Spikes pre:', M_pre.nspikes
print 'Spikes post:', M_post.nspikes

figure()
subplot(221)
imshow(weights_before, interpolation='nearest', origin='lower', aspect='auto')
pylab.gray()
subplot(222)
imshow(weights_after, interpolation='nearest', origin='lower', aspect='auto')
pylab.gray()
subplot(223)
plot(weights_before.squeeze(),'.')
subplot(224)
plot(weights_after.squeeze(),'.')

figure()
V = MV_post[0]
n = int(len(V)/repeats)
for j, i in enumerate(plotseq_startset):
    subplot(2,2,j+1)
    title(str(i))
    for k in range(plotseq_n):
        l = i+k
        plot(V[l*n:l*n+n])

figure()
subplot(121)
i, t = zip(*M_post.spikes)
t = array(t)-initial_segment_duration
t[t<0*ms]=0*ms
t[t>pattern_segment_duration]=0*ms
latency_reduction_t = t.copy()
plot(latency_reduction_t,'.')
title('Latency reduction (fig 5 in paper)')
subplot(122)
i, t = zip(*M_post.spikes)
m = int(len(t)/20)
if m==0: m=1
t = array(t)-initial_segment_duration
s = [t[n:n+m] for n in range(0,len(t),m)]
ps = [float(sum((si>0*ms) & (si<pattern_segment_duration)))/len(si) for si in s]
plot(ps)
title('Fraction of spikes within pattern')

show()



'''
Example of the use of the class :class:`~brian.hears.LinearGammachirp` available
in the library. It implements a filterbank of FIR gammatone filters with linear
frequency sweeps as described in Wagner et al. 2009, "Auditory responses in the
barn owl's nucleus laminaris to clicks: impulse response and signal analysis of
neurophonic potential", J. Neurophysiol. In this example, a white noise is
filtered by a gammachirp filterbank and the resulting cochleogram is plotted.
The different impulse responses are also plotted.
'''
from brian import *
from brian.hears import *

sound = whitenoise(100*ms).ramp()
sound.level = 50*dB

nbr_center_frequencies = 10  #number of frequency channels in the filterbank
#center frequencies with a spacing following an ERB scale
center_frequencies = erbspace(100*Hz, 1000*Hz, nbr_center_frequencies)

c = 0.0 #glide slope
time_constant = linspace(3, 0.3, nbr_center_frequencies)*ms

gamma_chirp = LinearGammachirp(sound, center_frequencies, time_constant, c) 

gamma_chirp_mon = gamma_chirp.process()

figure()

imshow(gamma_chirp_mon.T, aspect='auto')    
figure()
plot(gamma_chirp.impulse_response.T)
show()



from brian import *
from brian.utils.approximatecomparisons import *
from nose.tools import *

def test():
    reinit_default_clock()

    # PoissonGroup
    G = PoissonGroup(10, rates=10 * Hz)
    G1 = G[:5]
    G2 = G[5:]
    G1.rate = 5 * Hz
    G2.rate = 20 * Hz
    assert(is_approx_equal(G.rate[0], float(5 * Hz)))
    assert(is_approx_equal(G1.rate[0], float(5 * Hz)))
    assert(is_approx_equal(G2.rate[0], float(20 * Hz)))

if __name__ == '__main__':
    test()



from brian import *

#set_global_preferences(usecstdp=False)

G = NeuronGroup(10, 'V:1')
C = Connection(G, G, 'V')

eqs_stdp = Equations('''
dx/dt = -x/second : 1
dy/dt = -y/second : 1
du/dt = -u/second : 1
dv/dt = -v/second : 1
''')

stdp = STDP(C, eqs=eqs_stdp, 
            pre='x+=1; y+=1; w+=u+v',
            post='u+=1; v+=1; w+=x+y', 
            wmax=10)



"""
Spike queues following BEP-21
"""
from brian import *
from time import time

INITIAL_MAXSPIKESPER_DT = 1
# This is a 2D circular array, but also a SpikeMonitor

class SpikeQueue(SpikeMonitor):
    '''
    * Initialization * 

    Initialized with a source NeuronGroup, a Synapses object (from which it fetches the delays), a maximum delay
    
    Arguments
    ``source'' self explanatory
    ``synapses'' self explanatory

    Keywords
    ``max_delay'' in seconds
    ``maxevents'' Maximum initial number of events in each timestep. Notice that the structure will grow dynamically of there are more events than that, so you shouldn't bother. 


    * Circular 2D array structure * 
    
    A spike queue is implemented as a circular 2D array.
    
    * At the beginning or end of each timestep: queue.next()
    * To get all spikes: events=queue.peek()
      It returns the indexes of all synapses receiving an event.
    * When a presynaptic spike is emitted, the following is executed:
      queue.insert(delay,offset,target)
      where delay is the array of synaptic delays of targets in timesteps,
      offset is the array of offsets within each timestep,
      target is the array of synapse indexes of targets.
      The offset is used to solve the problem of multiple synapses with the
      same delay. For example, if there are two target synapses 7 and 9 with delay
      2 timesteps: queue.insert([2,2],[0,1],[7,9])
    
    Thus, offsets are determined by delays. They could be either precalculated
    (faster), or determined at run time (saves memory). Note that if they
    are determined at run time, then it may be possible to also vectorize over
    presynaptic spikes.
    
    * SpikeMonitor structure * 
    
    It automatically updates the underlying structure by instantiating the propagate() method of the SpikeMonitor
    
    Ideas:
    ------
    * remove the max_delay keyword and have the structure created with another
      method (at run time)
    '''
    def __init__(self, source, synapses, 
                 max_delay = 0, maxevents = INITIAL_MAXSPIKESPER_DT):
        # SpikeMonitor structure
        self.source = source #NeuronGroup
        self.synapses = synapses #Synapses
        
        self.max_delay = max_delay
        nsteps = int(np.floor((max_delay)/(self.source.clock.dt)))+1

        # number of time steps, maximum number of spikes per time step
        self.X = zeros((nsteps, maxevents), dtype = int) # target synapses
        self.X_flat = self.X.reshape(nsteps*maxevents,)
        self.currenttime = 0
        self.n = zeros(nsteps, dtype = int) # number of events in each time step
        
        super(SpikeQueue, self).__init__(source, 
                                         record = False)

    ################################ SPIKE QUEUE DATASTRUCTURE ######################
    def next(self):
        # Advance by one timestep
        self.n[self.currenttime]=0 # erase
        self.currenttime=(self.currenttime+1) % len(self.n)
        
    def peek(self):
        # Events in the current timestep
        return self.X[self.currenttime,:self.n[self.currenttime]]
    
    def offsets(self, delay):
        # Calculates offsets corresponding to a delay array
        #
        # Maybe it could be precalculated? (an int16 array?)
        I = argsort(delay)
        xs = delay[I]
        J = xs[1:]!=xs[:-1]
        #K = xs[1:]==xs[:-1]
        A = hstack((0, cumsum(J)))
        #B = hstack((0, cumsum(K)))
        B = hstack((0, cumsum(-J)))
        BJ = hstack((0, B[J]))
        ei = B-BJ[A]
        ofs = zeros_like(delay)
        ofs[I] = ei
        return ofs
        
    def insert(self, delay, offset, target):
        # Vectorized insertion of spike events
        # delay = delay in timestep
        # offset = offset within timestep
        # target = target synaptic index
        
        timesteps = (self.currenttime + delay) % len(self.n)
        
        # Compute new stack sizes:
        old_nevents = self.n[timesteps].copy() # because we need this for the final assignment, but we need to precompute the  new one to check for overflow
        self.n[timesteps] += offset+1 # that's a trick (to update stack size), plus we pre-compute it to check for overflow
        
        m = max(self.n[timesteps]) # If overflow, then at least one self.n is bigger than the size
        if (m >= self.X.shape[1]):
            self.resize(m)
        
        self.X_flat[(self.currenttime*self.X.shape[1]+offset+\
                     old_nevents)\
                     % len(self.X)]=target
        
    def resize(self, maxevents):
        '''
        Resizes the underlying data structure (number of columns = spikes per dt).
        max events will be rounded to the closest power of 2.
        '''
        
        # old and new sizes
        old_maxevents = self.X.shape[1]
        new_maxevents = 2**ceil(log2(maxevents))
        # new array
        newX = zeros((self.X.shape[0], new_maxevents), dtype = self.X.dtype)
        newX[:, :old_maxevents] = self.X[:, :old_maxevents] # copy old data
        
        self.X = newX
        self.X_flat = self.X.reshape(self.X.shape[0]*new_maxevents,)
        
        log_debug('spikequeue', 'Resizing SpikeQueue')
        
    def propagate(self, spikes):
        if len(spikes):
            # synapse identification, 
            # this seems ok in terms of speed even though I dont like the for loop. 
            # any idea? see stest_fastsynapseidentification.py
            synapses = []
            for i in spikes:
                # use hstack with arrays instead of lists
                synapses += list(nonzero(self.synapses._statevector._pre == i)[0]) # this is crazy!
                # we need a list/array of synapses for each presynaptic neuron
                # should we use array of arrays as objects?
                # and it could perhaps be on the SpikeQueue object

            if len(synapses):
                # delay getting:
                delay = self.synapses.delay[synapses]
                offsets = self.offsets(delay)
                self.insert(delay, offsets, synapses)
            
    ######################################## UTILS    
    def plot(self, display = True):
        for i in range(self.X.shape[0]):
            idx = (i + self.currenttime ) % self.X.shape[0]
            data = self.X[idx, :self.n[idx]]
            plot(idx * ones(len(data)), data, '.')
        if display:
            show()


    
'''
NOTE: the test code below is probably not working anymore since I changed the way it is created


The connection has arrays of synaptic variables (same as state matrix of
neuron groups). Two synaptic variables are the index of the postsynaptic neuron
and of the presynaptic neuron (i,j). (int32 or int16).

In addition, the connection must have, for each presynaptic neuron:
* list of target synapses (int32)
* corresponding delays in timesteps (int16)
* corresponding offsets (int16 is probably sufficient, or less)

These types (int32 etc) could be determined at construction time, or
at the time of conversion construction->connection (run time).

Same thing for postsynaptic neuron (for STDP)
This could also be determined at run time (depending on whether post=None or not)

Total memory:
* number of synapses * 12 * 2 (if bidirectional)
+ synaptic variables (weights)
'''

# We need to write some speed tests
if __name__=='__main__':
    queue=SpikeQueue(5,30)
    Nsynapses=4000*80 # in the CUBA example
    nspikes=160
    delays=randint(160,size=nspikes) # average number of spikes per dt in CUBA
    targets=randint(Nsynapses,size=nspikes)
    #print queue.offsets(delays)
    t1=time()
    for _ in range(10000): # 10000 timesteps per second
        d=queue.offsets(delays)
        queue.insert(delays,d,targets)
        queue.next()
        events=queue.peek()
        queue.plot()
    t2=time()
    print t2-t1



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

x = whitenoise(10*ms)

fb = GammatoneFilterbank(x, [10*Hz, 20*Hz])

fb2 = Repeat(fb, 3)

for k, v in fb.__dict__.iteritems():
    if 'buffer' in k and not isinstance(v, ndarray):
        print k, ':', v

print '***'

fb.buffer_fetch(0, len(x))

for k, v in fb.__dict__.iteritems():
    if 'buffer' in k and not isinstance(v, ndarray):
        print k, ':', v

fb2.buffer_fetch(0, len(x))



'''
Example of using :class:`RemoteControlServer` and :class:`RemoteControlClient`
to control a simulation as it runs in Brian.

Run the script remotecontrolserver.py before running this.
'''
from brian import *
import time

client = RemoteControlClient()

time.sleep(1)

subplot(121)
plot(*client.evaluate('(M.times, M.values)'))

client.execute('G.I = 1.1')

time.sleep(1)

subplot(122)
plot(*client.evaluate('(M.times, M.values)'))

client.stop()

show()



from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection
from brian.tools.statistics import firing_rate, get_gamma_factor
try:
    from playdoh import *
except Exception, e:
    print e
    raise ImportError("Playdoh must be installed (https://code.google.com/p/playdoh/)")

try:
    import pycuda
    from gpu_modelfitting import GPUModelFitting
    can_use_gpu = True
except ImportError:
    can_use_gpu = False
from brian.experimental.codegen.integration_schemes import *
from criteria import *
from simulator import * 
import sys, cPickle

__all__ = ['modelfitting', 'print_table', 'PSO', 'GA', 'CMAES',
           'slice_trace', 'transform_spikes',
           'MAXCPU', 'MAXGPU',
           'GammaFactor','GammaFactor2', 'LpError','VanRossum','Brette',
           'simulate', 
           'debug_level', 'info_level', 'warning_level', 'open_server']




class ModelFitting(Fitness):
    def initialize(self, **kwds):
        
        # Gets the key,value pairs in shared_data
        for key, val in self.shared_data.iteritems():
            setattr(self, key, val)
            
        # Gets the key,value pairs in **kwds
        for key, val in kwds.iteritems():
            setattr(self, key, val)
        
        self.model = cPickle.loads(self.model)
        if type(self.model) is str:
            self.model = Equations(self.model)
        
        self.simulator = Simulator(self.model, self.reset, self.threshold,
                                   inputs = self.inputs, input_var = self.input_var,
                                   dt = self.dt,
                                   refractory = self.refractory,
                                   max_refractory = self.max_refractory,
                                   spikes = self.spikes,
                                   traces = self.traces,
                                   groups = self.groups,
                                   slices = self.slices,
                                   overlap = self.overlap,
                                   onset = self.onset,
                                   neurons = self.nodesize,
                                   initial_values = self.initial_values,
                                   unit_type = self.unit_type,
                                   stepsize = self.stepsize,
                                   precision = self.precision,
                                   criterion = self.criterion,
                                   ntrials=self.ntrials,
                                   method = self.method
                                   )
            
    def evaluate(self, **param_values):
        """
        Use fitparams['delays'] to take delays into account
        Use fitparams['refractory'] to take refractory into account
        """
        values = self.simulator.run(**param_values)
        return values




def modelfitting(model=None,
                 reset=None,
                 threshold=None,
                 refractory=0*ms,
                 data=None,
                 input_var='I',
                 input=None,
                 dt=None,
                 popsize=1000,
                 maxiter=10,
                 slices=1,
                 overlap=None,
                 onset=None,
                 initial_values=None,
                 stepsize=100 * ms,
                 unit_type=None,
                 total_units=None,
                 ntrials=1,
                 cpu=None,
                 gpu=None,
                 precision='double', # set to 'float' or 'double' to specify single or double precision on the GPU
                 machines=[],
                 allocation=None,
                 returninfo=False,
                 scaling=None,
                 algorithm=CMAES,
                 async = None,
                 criterion=None,
                 optparams={},
                 method='Euler',
                 **params):
    """
    Model fitting function.
    
    Fits a spiking neuron model to electrophysiological data (injected current and spikes).
    
    See also the section :ref:`model-fitting-library` in the user manual.
    
    **Arguments**
    
    ``model``
        An :class:`~brian.Equations` object containing the equations defining the model.
    
    ``reset``
        A reset value for the membrane potential, or a string containing the reset
        equations.
        
    ``threshold``
        A threshold value for the membrane potential, or a string containing the threshold
        equations.
        
    ``refractory``
        The refractory period in second. If it's a single value, the same refractory will be
        used in all the simulations. If it's a list or a tuple, the fitting will also
        optimize the refractory period (see ``**params`` below).
        
        Warning: when using a refractory period, you can't use a custom reset, only a fixed one.
        
    ``data``
        A list of spike times, or a list of several spike trains as a list of pairs (index, spike time)
        if the fit must be performed in parallel over several target spike trains. In this case,
        the modelfitting function returns as many parameters sets as target spike trains.
        
    ``input_var='I'``
        The variable name used in the equations for the input current.
        
    ``input``
        A vector of values containing the time-varying signal the neuron responds to (generally
        an injected current).
        
    ``dt``
        The time step of the input (the inverse of the sampling frequency).
        
    ``**params``
        The list of parameters to fit the model with. Each parameter must be set as follows:
        ``param_name=[bound_min, min, max, bound_max]``
        where ``bound_min`` and ``bound_max`` are the boundaries, and ``min`` and ``max``
        specify the interval from which the parameter values are uniformly sampled at
        the beginning of the optimization algorithm.
        If not using boundaries, set ``param_name=[min, max]``.
        
        Also, you can add a fit parameter which is a spike delay for all spikes :
        add the special parameter ``delays`` in ``**params``, for example 
        ``modelfitting(..., delays=[-10*ms, 10*ms])``.
        
        You can also add fit the refractory period by specifying 
        ``modelfitting(..., refractory=[-10*ms, 10*ms])``.
        
    ``popsize``
        Size of the population (number of particles) per target train used by the optimization algorithm.
        
    ``maxiter``
        Number of iterations in the optimization algorithm.
        
    ``optparams``
        Optimization algorithm parameters. It is a dictionary: keys are parameter names,
        values are parameter values or lists of parameters (one value per group). 
        This argument is specific to the optimization
        algorithm used. See :class:`PSO`, :class:`GA`, :class:`CMAES`. 
        
    ``delta=4*ms``
        The precision factor delta (a scalar value in second).
        
    ``slices=1``
        The number of time slices to use.
        
    ``overlap=0*ms``
        When using several time slices, the overlap between consecutive slices, in seconds.
        
    ``initial_values``
        A dictionary containing the initial values for the state variables.
        
    ``cpu``
        The number of CPUs to use in parallel. It is set to the number of CPUs in the machine by default.
        
    ``gpu``
        The number of GPUs to use in parallel. It is set to the number of GPUs in the machine by default.
        
    ``precision``
        GPU only: a string set to either ``float`` or ``double`` to specify whether to use
        single or double precision on the GPU. If it is not specified, it will
        use the best precision available.
        
    ``returninfo=False``
        Boolean indicating whether the modelfitting function should return technical information
        about the optimization.
        
    ``scaling=None``
        Specify the scaling used for the parameters during the optimization. 
        It can be ``None`` or ``'mapminmax'``. It is ``None``
        by default (no scaling), and ``mapminmax`` by default for the CMAES algorithm.
        
    ``algorithm=CMAES``
        The optimization algorithm. It can be :class:`PSO`, :class:`GA` or :class:`CMAES`.
         
    ``optparams={}``
         Optimization parameters. See
         
    ``method='Euler'``
        Integration scheme used on the CPU and GPU: ``'Euler'`` (default), ``RK``, 
        or ``exponential_Euler``.
        See also :ref:`numerical-integration`.
        
    ``machines=[]``
        A list of machine names to use in parallel. See :ref:`modelfitting-clusters`.
    
    **Return values**
    
    Return an :class:`OptimizationResult` object with the following attributes:
    
    ``best_pos``
        Minimizing position found by the algorithm. For array-like fitness functions,
        it is a single vector if there is one group, or a list of vectors.
        For keyword-like fitness functions, it is a dictionary
        where keys are parameter names and values are numeric values. If there are several groups,
        it is a list of dictionaries.
    
    ``best_fit``
        The value of the fitness function for the best positions. It is a single value if 
        there is one group, or it is a list if there are several groups.
    
    ``info``
        A dictionary containing various information about the optimization.


    Also, the following syntax is possible with an ``OptimizationResult`` instance ``or``.
    The ``key`` is either an optimizing parameter name for keyword-like fitness functions,
    or a dimension index for array-like fitness functions.
    
    ``or[key]``
        it is the best ``key`` parameter found (single value), or the list
        of the best parameters ``key`` found for all groups.
    
    ``or[i]``
        where ``i`` is a group index. This object has attributes ``best_pos``, ``best_fit``,
        ``info`` but only for group ``i``.
    
    ``or[i][key]``
        where ``i`` is a group index, is the same as ``or[i].best_pos[key]``.

    For more details on the gamma factor, see
    `Jolivet et al. 2008, "A benchmark test for a quantitative assessment of simple neuron models", J. Neurosci. Methods <http://www.ncbi.nlm.nih.gov/pubmed/18160135>`__ (available in PDF
    `here <http://icwww.epfl.ch/~gerstner/PUBLICATIONS/Jolivet08.pdf>`__).
    """
    
    for param in params.keys():
        if (param not in model._diffeq_names) and (param != 'delays') and (param != 'tau_metric'):
            raise Exception("Parameter %s must be defined as a parameter in the model" % param)
    
    if criterion is None:
        criterion = GammaFactor()
        
    data = array(data)
    if criterion.type == 'spikes':
        # Make sure that 'data' is a N*2-array
        if data.ndim == 1:
            data = concatenate((zeros((len(data), 1)), data.reshape((-1, 1))), axis=1)
        spikes = data
        traces = None
        if ntrials>1:
            groups = 1
        else:
            groups = int(array(data)[:, 0].max() + 1) # number of target trains

    elif criterion.type == 'trace':
        if data.ndim == 1:
            data = data.reshape((1,-1))
        spikes = None
        traces = data
        groups = data.shape[0]
    elif criterion.type == 'both':
        # TODO
        log_warn("Not implemented yet")
        pass
    inputs = input
    if inputs.ndim==1:
        inputs = inputs.reshape((1,-1))

    # dt must be set
    if dt is None:
        raise Exception('dt (sampling frequency of the input) must be set')

    # default overlap when no time slicing
    if overlap is None:
        overlap = 0*ms
    if onset is None:
        onset = overlap
#    if slices == 1:
#        overlap = 0*ms
#        onset = overlap
        
    # default allocation
    if cpu is None and gpu is None and unit_type is None:
        if CANUSEGPU: unit_type = 'GPU'
        else: unit_type = 'CPU'

    # check numerical integration method
    if (gpu>0 or unit_type == 'GPU') and method not in ['Euler', 'RK', 'exponential_Euler']:
        raise Exception("The method can only be 'Euler', 'RK', or 'exponential_Euler' when using the GPU") 
    if method not in ['Euler', 'RK', 'exponential_Euler', 'linear', 'nonlinear']:
        raise Exception("The method can only be 'Euler', 'RK', 'exponential_Euler', 'linear', or 'nonlinear'")

    if (algorithm == CMAES) & (scaling is None):
        scaling = 'mapminmax'
        
    # determines whether optimization over refractoriness or not
    if type(refractory) is tuple or type(refractory) is list:
        params['refractory'] = refractory
        max_refractory = refractory[-1]
    else:
        max_refractory = None
    
#    duration = len(input) * dt # duration of the input

    # keyword arguments for Modelfitting initialize
    kwds = dict(   model=cPickle.dumps(model),
                   threshold=threshold,
                   reset=reset,
                   refractory=refractory,
                   max_refractory=max_refractory,
                   input_var=input_var, dt=dt,
                   criterion=criterion,
                   slices=slices,
                   overlap=overlap,
                   returninfo=returninfo,
                   precision=precision,
                   stepsize=stepsize,
                  ntrials=ntrials,
                   method=method,
                   onset=onset)

    shared_data = dict(inputs=inputs,
                       traces=traces,
                       spikes=spikes,
                       initial_values=initial_values)

    if async:
        r = maximize_async( ModelFitting,
                            shared_data=shared_data,
                            kwds = kwds,
                            groups=groups,
                            popsize=popsize,
                            maxiter=maxiter,
                            optparams=optparams,
                            unit_type = unit_type,
                            machines=machines,
                            allocation=allocation,
                            total_units = total_units,
                            cpu=cpu,
                            gpu=gpu,
                            returninfo=returninfo,
                            codedependencies=[],
                            algorithm=algorithm,
                            scaling=scaling,
                            **params)
    else:
        r = maximize(       ModelFitting,
                            shared_data=shared_data,
                            kwds = kwds,
                            groups=groups,
                            popsize=popsize,
                            maxiter=maxiter,
                            optparams=optparams,
                            unit_type = unit_type,
                            machines=machines,
                            allocation=allocation,
                            total_units = total_units,
                            cpu=cpu,
                            gpu=gpu,
                            returninfo=returninfo,
                            codedependencies=[],
                            algorithm=algorithm,
                            scaling=scaling,
                            **params)

    # r is (results, fitinfo) or (results)
    return r




#def get_spikes(model=None, reset=None, threshold=None,
#                input=None, input_var='I', dt=None,
#                **params):
#    """
#    Retrieves the spike times corresponding to the best parameters found by
#    the modelfitting function.
#    
#    **Arguments**
#    
#    ``model``, ``reset``, ``threshold``, ``input``, ``input_var``, ``dt``
#        Same parameters as for the ``modelfitting`` function.
#        
#    ``**params``
#        The best parameters returned by the ``modelfitting`` function.
#    
#    **Returns**
#    
#    ``spiketimes``
#        The spike times of the model with the given input and parameters.
#    """
#    duration = len(input) * dt
#    ngroups = len(params[params.keys()[0]])
#
#    group = NeuronGroup(N=ngroups, model=model, reset=reset, threshold=threshold,
#                        clock=Clock(dt=dt))
#    group.set_var_by_array(input_var, TimedArray(input, clock=group.clock))
#    for param, values in params.iteritems():
#        if (param == 'delays') | (param == 'fitness'):
#            continue
#        group.state(param)[:] = values
#
#    M = SpikeMonitor(group)
#    net = Network(group, M)
#    net.run(duration)
#    reinit_default_clock()
#    return M.spikes
#
#def predict(model=None, reset=None, threshold=None,
#            data=None, delta=4 * ms,
#            input=None, input_var='I', dt=None,
#            **params):
#    """
#    Predicts the gamma factor of a fitted model with respect to the data with
#    a different input current.
#    
#    **Arguments**
#    
#    ``model``, ``reset``, ``threshold``, ``input_var``, ``dt``
#        Same parameters as for the ``modelfitting`` function.
#        
#    ``input``
#        The input current, that can be different from the current used for the fitting
#        procedure.
#    
#    ``data``
#        The experimental spike times to compute the gamma factor against. They have
#        been obtained with the current ``input``.
#    
#    ``**params``
#        The best parameters returned by the ``modelfitting`` function.
#    
#    **Returns**
#    
#    ``gamma``
#        The gamma factor of the model spike trains against the data.
#        If there were several groups in the fitting procedure, it is a vector
#        containing the gamma factor for each group.
#    """
#    spikes = get_spikes(model=model, reset=reset, threshold=threshold,
#                        input=input, input_var=input_var, dt=dt,
#                        **params)
#
#    ngroups = len(params[params.keys()[0]])
#    gamma = zeros(ngroups)
#    for i in xrange(ngroups):
#        spk = [t for j, t in spikes if j == i]
#        gamma[i] = gamma_factor(spk, data, delta, normalize=True, dt=dt)
#    if len(gamma) == 1:
#        return gamma[0]
#    else:
#        return gamma




if __name__ == '__main__':
    
    from brian import loadtxt, ms, savetxt, loadtxt, Equations, NeuronGroup, run, SpikeMonitor,\
         StateMonitor, Network
    from pylab import *
    
    def generate_data():
        g = NeuronGroup(1, model=equations, reset=0, threshold=1)
        g.I = TimedArray(input, dt=.1*ms)
        g.tau = 25*ms
        g.R = 3e9
        SpM = SpikeMonitor(g)
        StM = StateMonitor(g, 'V', record=True)
        net = Network(g, SpM, StM)
        net.run(1*second)
        return StM.values[0], SpM.spikes
    
    equations = Equations('''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
        #tau_metric : second
    ''')
    input = loadtxt('current.txt')
    
    # ARTIFICIAL DATA: R=3e9, tau=25*ms
#    spikes = loadtxt('spikes.txt') # real data
    trace, spikes = generate_data()
#    savetxt('trace_artificial.txt', trace)
#    savetxt('spikes_artificial.txt', spikes)
    
#    trace = loadtxt('trace_artificial.txt')
    overlap = 10*ms
    slices = 1
    dt = .1*ms
    input, trace = slice_trace(input, trace, slices = slices, dt=dt, overlap = overlap)
    
    # GAMMA FACTOR
#    criterion = GammaFactor(delta=2*ms)
#    spikes= loadtxt('spikes_artificial.txt')
#    data = spikes
#    data[:,1] += 50*ms
    
#    # LP ERROR
    criterion = LpError(p=2, varname='V')
    data = trace
    print data.shape
    print input.shape 
#    # Van Rossum
#    criterion = VanRossum(tau=2*ms)
#    data = spikes
    
    #Brette
#    criterion = Brette()
#    data = spikes

    results = modelfitting( model = equations,
                            reset = 0,
                            threshold = 1,
                            data = data,
                            input = input,
                            #onset = overlap,
                            gpu = 1,
                            #cpu=4,
                            dt = dt,
                            popsize = 20000,
                            maxiter = 10,
                            criterion = criterion,
                            R = [1.0e9,1.0e9, 9.0e9, 9.0e9],
                            tau = [10*ms,10*ms, 40*ms, 40*ms],
                            algorithm=CMAES,
                            tau_metric= [0.5*ms,0.5*ms, 4*ms, 4*ms],
                            #delays=[-1*ms, 1*ms]
                            )
    print_table(results)
    
#    criterion_values, record_values = simulate( model = equations,
#                                                reset = 0,
#                                                threshold = 1,
#                                                data = data,
#                                                input = input,
#                                                use_gpu = False,
#                                                dt = dt,
#                                                criterion = criterion,
#                                                record = 'V',
#                                                onset = overlap,
#                                                neurons = slices,
#                                                **results.best_params
#                                                )
#    
#    trace = trace[:,int(overlap/dt):].flatten()
#    traceV = record_values[:,int(overlap/dt):].flatten()
#    
#    plot(trace)
#    plot(traceV)
#    
#    show()
#    


'''
Input-Frequency curve of a IF model
Network: 1000 unconnected integrate-and-fire neurons (leaky IF)
with an input parameter v0.
The input is set differently for each neuron.
Spikes are sent to a spike counter (counts the spikes emitted by each neuron).
'''
from brian import *

N = 1000
tau = 10 * ms
eqs = '''
dv/dt=(v0-v)/tau : volt
v0 : volt
'''
group = NeuronGroup(N, model=eqs, threshold=10 * mV, reset=0 * mV, refractory=5 * ms)
group.v = 0 * mV
group.v0 = linspace(0 * mV, 20 * mV, N)

counter = SpikeCounter(group)

duration = 5 * second
run(duration)
plot(group.v0 / mV, counter.count / duration)
show()



'''
Example showing how to fit a single model
with different target spike trains (several groups).
'''
from brian import loadtxt, ms, Equations, second
from brian.library.modelfitting import *

if __name__ == '__main__':

    model = Equations('''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
    ''')

    input = loadtxt('current.txt')
    spikes0 = loadtxt('spikes.txt')
    spikes = []
    for i in xrange(2):
        spikes.extend([(i, spike*second + 5*i*ms) for spike in spikes0])

    results = modelfitting( model = model,
                            reset = 0,
                            threshold = 1,
                            data = spikes,
                            input = input,
                            dt = .1*ms,
                            popsize = 1000,
                            maxiter = 3,
                            cpu = 1,
                            delta = 4*ms,
                            R = [1.0e9, 9.0e9],
                            tau = [10*ms, 40*ms],
                            delays = [-10*ms, 10*ms])
    print_table(results)




from brian import *

P = NeuronGroup(100, model='dv/dt=0*Hz:1')
Q = NeuronGroup(200, model='dv/dt=0*Hz:1')
C = Connection(P, Q, 'v')
C.connect_random(P, Q, sparseness=lambda i, j:exp(-.05 * abs(i - j)), weight=lambda i, j:abs(cos(.1 * (i - j))))
C2 = Connection(P, Q, 'v')
C2.connect_random(P, Q, sparseness=lambda i, j:exp(-.05 * abs(i - j)), weight=lambda: rand())

print C.W.nnz, C2.W.nnz

figure()
subplot(211)
imshow(C.W.todense(), interpolation='nearest', origin='lower')
subplot(212)
imshow(C2.W.todense(), interpolation='nearest', origin='lower')
show()



'''
Very short example program.
'''
from brian import *

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)

Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)

M = SpikeMonitor(P)

run(1 * second)
i = 0
while len(M[i]) <= 1:
    i += 1
print "The firing rate of neuron", i, "is", firing_rate(M[i]) * Hz
print "The coefficient of variation neuron", i, "is", CV(M[i])
raster_plot(M)
show()



'''
Example of the use of the class :class:`~brian.hears.ApproximateGammatone`
available in the library. It implements a filterbank of approximate gammatone
filters as  described in Hohmann, V., 2002, "Frequency analysis and synthesis
using a Gammatone filterbank", Acta Acustica United with Acustica. 
In this example, a white noise is filtered by a gammatone filterbank and the
resulting cochleogram is plotted.
'''
from brian import *
from brian.hears import *

level=50*dB # level of the input sound in rms dB SPL
sound = whitenoise(100*ms).ramp() # generation of a white noise
sound = sound.atlevel(level) # set the sound to a certain dB level

nbr_center_frequencies = 50  # number of frequency channels in the filterbank
# center frequencies with a spacing following an ERB scale
center_frequencies = erbspace(100*Hz, 1000*Hz, nbr_center_frequencies)
# bandwidth of the filters (different in each channel) 
bw = 10**(0.037+0.785*log10(center_frequencies))

gammatone = ApproximateGammatone(sound, center_frequencies, bw, order=3) 

gt_mon = gammatone.process()

figure()
imshow(flipud(gt_mon.T), aspect='auto')    
show()



'''
Spike-timing dependent plasticity
Adapted from Fig. 3 in Song and Abbott (2001)

About 10 times slower than real time.

'''
from brian import *
from time import time

Nin = 1000
Nout = 200
taum = 20 * ms
tau_pre = 20 * ms
tau_post = tau_pre
Ee = 0 * mV
vt = -54 * mV
vr = -60 * mV
vrest = -74 * mV
taue = 5 * ms
gmax = .02
dA_pre = .001*5
dA_post_ff = -dA_pre * tau_pre / tau_post * 1.06
dA_post_rec = -dA_pre * tau_pre / tau_post * 1.04
R0=10*Hz
R1=80*Hz
sigma=100.
stimulus_period=20*ms
Fext=150*Hz
gext=0.096

input_rate=lambda s,a:R0+R1*(exp(-(s-a)**2/(2.*sigma**2))+exp(-(s+1000.-a)**2/(2.*sigma**2))+exp(-(s-1000.-a)**2/(2.*sigma**2)))
all_inputs=arange(Nin)

eqs_neurons = '''
dv/dt=(ge*(Ee-v)+vrest-v)/taum : volt   # the synaptic current is linearized
dge/dt=-ge/taue : 1
'''

input = PoissonGroup(Nin, rates=input_rate(all_inputs,500))
input_ext=PoissonGroup(Nout,rates=Fext)
neurons = NeuronGroup(Nout, model=eqs_neurons, threshold=vt, reset=vr)
synapses_ff = Connection(input, neurons, 'ge', weight=lambda: rand()*gmax, sparseness=0.2)
#synapses_ff.W.alldata=rand(synapses_ff.W.nnz)*gmax
#synapses_rec = Connection(neurons, neurons, 'ge', weight=0, structure='dense')
synapses_ext = IdentityConnection(input_ext,neurons,'ge',weight=gext)
neurons.v = vrest

stdp_ff=ExponentialSTDP(synapses_ff,tau_pre,tau_post,dA_pre,dA_post_ff,wmax=gmax)
#stdp_rec=ExponentialSTDP(synapses_rec,tau_pre,tau_post,dA_pre,dA_post_rec,wmax=gmax)

next_change=exponential()*stimulus_period # maybe define a new clock?

@network_operation
def new_input(cl):
    global next_change
    if cl.t>=next_change: # new input
        input.rate=input_rate(all_inputs,randint(Nin))
        next_change=exponential()*stimulus_period

rate = PopulationRateMonitor(neurons)

start_time = time()
run(30 * second, report='text')
print "Simulation time:", time() - start_time

figure()
subplot(311)
plot(rate.times / second, rate.smooth_rate(100 * ms))
subplot(312)
hist(synapses_ff.W.alldata / gmax, 20)

figure()
imshow(synapses_ff.W.todense()/gmax)
colorbar()

#figure()
#imshow(synapses_rec.W.todense()/gmax)
#colorbar()

show()



from brian import *
import brian
import os

def go():
    try:
        import nose
    except ImportError:
        print "Brian testing framework uses the 'nose' package."
    print 'Brian running from file:', brian.__file__
    # For running tests from an IPython shell, use magic_useframes=True, but we
    # restore the state after running 
    magic_useframes = get_global_preference('magic_useframes')
    set_global_preferences(magic_useframes=True)
    nose.config.logging.disable(nose.config.logging.ERROR)
    basedir, _ = os.path.split(__file__)
    cwd = os.getcwd()
    os.chdir(basedir)
    nose.run()
    os.chdir(cwd)
    set_global_preferences(magic_useframes=magic_useframes)

if __name__ == '__main__':
    go()



'''
Reliability of spike timing.
See e.g. Mainen & Sejnowski (1995) for experimental results in vitro.

R. Brette
'''
from brian import *

# The common noisy input
N = 25
tau_input = 5 * ms
input = NeuronGroup(1, model='dx/dt=-x/tau_input+(2./tau_input)**.5*xi:1')

# The noisy neurons receiving the same input
tau = 10 * ms
sigma = .015
eqs_neurons = '''
dx/dt=(0.9+.5*I-x)/tau+sigma*(2./tau)**.5*xi:1
I : 1
'''
neurons = NeuronGroup(N, model=eqs_neurons, threshold=1, reset=0, refractory=5 * ms)
neurons.x = rand(N)
neurons.I = linked_var(input,'x') # input.x is continuously fed into neurons.I
spikes = SpikeMonitor(neurons)

run(500 * ms)
raster_plot(spikes)
show()



from brian import *
from nose.tools import *
from operator import itemgetter
from brian.utils.approximatecomparisons import is_approx_equal

def test():
    """
    :class:`Threshold`
    ~~~~~~~~~~~~~~~~~~
    
    Initialised as ``Threshold(threshold[,state=0])``
    
    Causes a spike whenever the given state variable is above
    the threshold value.
    
    :class:`NoThreshold`
    ~~~~~~~~~~~~~~~~~~~~
    
    Does nothing, initialised as ``NoThreshold()``
    
    Functional thresholds
    ~~~~~~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        FunThreshold(thresholdfun)
        SimpleFunThreshold(thresholdfun[,state=0])
    
    Threshold functions return a boolean array the same size as the
    number of neurons in the group, where if the returned array is
    True at index i then neuron i fires.
    
    The arguments passed to the :class:`FunThreshold` function are the
    full array of state variables for the group in order.
    
    The argument passed to the :class:`SimpleFunThreshold` function is the
    array of length N corresponding to the given state variable.
    
    :class:`VariableThreshold`
    ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Initialised as ``VariableThreshold(threshold_state[,state=0])``
    
    Causes a spike whenever the state variable defined by state
    is above the state variable defined by threshold_state.
    
    :class:`EmpiricalThreshold`
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        EmpiricalThreshold(threshold[,refractory=1*msecond[,state=0[,clock]]])
    
    Causes a spike when the given state variable exceeds the threshold value,
    but only if there has been no spike within the refractory period. Will
    use the given clock if specified, otherwise the standard guessing procedure
    is used.
    
    Poisson thresholds
    ~~~~~~~~~~~~~~~~~~
    
    Initialised as::
    
        PoissonThreshold([state=0])
        HomogeneousPoissonThreshold([state=0])
    
    The Poisson process gets the rates from the specified state variable, the
    homogeneous version uses the rates from the specified variable of the first
    neuron in the group.      
    """
    reinit_default_clock()

    # test that Threshold works as expected with default state
    G = NeuronGroup(3, model=LazyStateUpdater(), reset=Reset(0.), threshold=Threshold(1.), init=(0.,))
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    net.run(1 * msecond)
    assert (len(M.spikes) == 0)
    G.state(0)[:] = array([0.5, 1.5, 2.5])
    net.run(1 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(0)))
    assert (i == (1, 2))
    for s in t: assert (is_approx_equal(s, 1 * msecond))

    # test that Threshold works as expected with specified state
    G = NeuronGroup(3, model=LazyStateUpdater(numstatevariables=2), reset=Reset(0., state=1), threshold=Threshold(1., state=1), init=(0., 0.))
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    net.run(1 * msecond)
    assert (len(M.spikes) == 0)
    net.reinit()
    G.state(0)[:] = array([0.5, 1.5, 2.5])
    net.run(1 * msecond)
    assert (len(M.spikes) == 0)
    net.reinit()
    G.state(1)[:] = array([0.5, 1.5, 2.5])
    net.run(1 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(0)))
    assert (i == (1, 2))
    for s in t: assert (is_approx_equal(s, 0 * msecond))

    # test that VariableThreshold works as expected
    G = NeuronGroup(3, model=LazyStateUpdater(numstatevariables=3), reset=Reset(0., state=1), threshold=VariableThreshold(2, state=1), init=(0., 0., 0.))
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    get_default_clock().reinit()
    G.state(2)[:] = array([1., 2., 3.]) # the thresholds
    G.state(1)[:] = array([4., 1., 2.]) # the values
    net.run(1 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(0)))
    assert (i == (0,))
    assert (is_approx_equal(t[0], 0 * second))

    # test that FunThreshold works as expected
    def f(S0, S1):
        return S0 > S1 * S1
    G = NeuronGroup(3, model=LazyStateUpdater(numstatevariables=2), reset=Reset(0.), threshold=FunThreshold(f), init=(0., 0.))
    G.state(0)[:] = array([2., 3., 10.])
    G.state(1)[:] = array([1., 2., 3.]) # the square root of the threshold values
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    get_default_clock().reinit()
    net.run(1 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(0)))
    assert (i == (0, 2))
    for s in t: assert (is_approx_equal(s, 0 * msecond))

    # test that SimpleFunThreshold works as expected
    def f(S):
        return S > 1.
    G = NeuronGroup(3, model=LazyStateUpdater(), reset=Reset(0.), threshold=SimpleFunThreshold(f), init=(0.,))
    G.state(0)[:] = array([0.5, 1.5, 2.5])
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    get_default_clock().reinit()
    net.run(1 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(0)))
    assert (i == (1, 2))
    for s in t: assert (is_approx_equal(s, 0 * msecond))

    # test that EmpiricalThreshold works as expected
    G = NeuronGroup(1, model=LazyStateUpdater(numstatevariables=2), reset=NoReset(), threshold=EmpiricalThreshold(1., refractory=0.5 * msecond, state=1), init=(0., 2.))
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    get_default_clock().reinit()
    net.run(1.6 * msecond)
    i, t = zip(*sorted(M.spikes, key=itemgetter(1)))
    assert (i == (0, 0, 0, 0))
    for i, s in enumerate(t): assert (is_approx_equal(s, i * 0.5 * msecond))

    # test that PoissonThreshold works
    init = float(1. / get_default_clock().dt) # should cause spiking at every time interval
    G = NeuronGroup(3, model=LazyStateUpdater(), reset=NoReset(), threshold=PoissonThreshold())
    G.state(0)[:] = array([0., init, 0.])
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    net.run(1 * msecond)
    assert (len(M.spikes))
    i, t = zip(*sorted(M.spikes, key=itemgetter(1)))
    assert (all(j == 1 for j in i))

    # test that HomogeneousPoissonThreshold works
    init = float(1. / get_default_clock().dt) # should cause spiking at every time interval
    G = NeuronGroup(3, model=LazyStateUpdater(), reset=NoReset(), threshold=HomogeneousPoissonThreshold())
    M = SpikeMonitor(G, True)
    net = Network(G, M)
    G.state(0)[:] = array([0., init, 0.]) # should do nothing, because only first neuron is looked at 
    net.run(1 * msecond)
    assert (len(M.spikes) == 0)
    G.state(0)[:] = array([init, 0., 0.]) # should do nothing, because only first neuron is looked at 
    net.run(1 * msecond)
    # we actually cannot make any assertion about the behaviour of this system, other than
    # that it should run correctly    

if __name__ == '__main__':
    test()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Currents - Objects that can be plugged into membrane equations
'''
import types
from operator import isSequenceType
from stateupdater import StateUpdater
from units import *

__all__ = ['find_capacitance', 'Current', 'exp_current', 'exp_conductance', \
         'leak_current']

def find_capacitance(model):
    '''
    Tries to find the membrane capacitance from
    a set of differential equations or a model given as
    a dictionnary (with typical keys 'model', 'threshold' and 'reset').
    '''
    if type(model) == types.DictType:
        if 'Cm' in model:
            return model['Cm']
        if 'C' in model:
            return model['C']
        # Failed: look the equations
        if 'model' in model:
            model = model['model']
        else: # no clue!
            raise TypeError, 'Strange model!'

    if isinstance(model, StateUpdater):
        if hasattr(model, 'Cm'):
            return model.Cm
        if hasattr(model, 'C'):
            return model.C

    if isSequenceType(model):
        model = model[0] # The first equation is the membrane equation
    if type(model) == types.FunctionType:
        if 'Cm' in model.func_globals:
            return model.func_globals['Cm']
        if 'C' in model.func_globals:
            return model.func_globals['C']

    # Nothing was found!
    raise TypeError, 'No capacitance found!'


class Current(object):
    '''
    A membrane current.
    '''
    def __init__(self, I=lambda v:0, eqs=[]):
        '''
        I = current function, e.g. I=lambda v,g: g*(v-E)
        eqs= differential system, e.g. eqs=[lambda v,g:-g]
        '''
        self.I = I
        self.eqs = eqs

    def __radd__(self, model):
        '''
        model + self: addition of a current to a membrane equation
        '''
        Cm = find_capacitance(model)
        # Dictionnary?
        modeldict = None
        if type(model) == types.DictType:
            modeldict = model
            model = model['model']
        # Only one function?
        if type(model) == types.FunctionType:
            model = [model]
        # Assume sequence type
        n = len(model) # number of equations in model
        m = len(self.eqs) # number of equations in current
        print n, m
        newmodel = [0] * (n + m)
        # Insert current in membrane equation
        newmodel[0] = lambda * args: model[0](*(args[0:n])) + \
                    self.I(args[0], *args[n:n + m]) / Cm
        # Adjust the number of variables
        # Tricky bit here
        for i in range(1, n):
            md = model[i]
            newmodel[i] = lambda * args: md(*(args[0:n]))
        #newmodel[1:n]=[lambda *args: md(*(args[0:n])) for md in model[1:n]]
        # Add current variables
        # Adjust the number of variables
        for i in range(n, n + m):
            md = self.eqs[i - n]
            newmodel[i] = lambda * args: md(args[0], *(args[n:n + m]))
        #newmodel[n:n+m]=[lambda *args: md(args[0],*(args[n:n+m])) for md in self.eqs[n:n+m]]        
        # Dictionnary?
        if modeldict != None:
            modeldict['model'] = newmodel
            return modeldict
        else:
            return newmodel

# ---------------------
# Synaptic currents
# TODO: alpha and biexp
# ---------------------
@check_units(tau=second)
def exp_current(tau):
    '''
    Exponential current.
      I=g
      dg=-g/tau
    '''
    return Current(I=lambda v, g:g, eqs=[lambda v, g:-g / tau])

@check_units(tau=second, E=volt)
def exp_conductance(tau, E):
    '''
    Exponential conductance.
      I=g*(E-v)
      dg=-g/tau
    '''
    return Current(I=lambda v, g:g * (E - v), eqs=[lambda v, g:-g / tau])

# ------------------
# Intrinsic currents
# ------------------
@check_units(gl=siemens, El=volt)
def leak_current(gl, El):
    '''
    Leak current.
      I=gl*(El-v)
    '''
    return Current(I=lambda v:gl * (v - El), eqs=[])

# Test
if __name__ == '__main__':
    C = 2.
    dv = lambda v:-v / C
    #mymodel={'model':[dv],'Cm':3}
    m = dv + exp_conductance(10 * msecond, E= -70 * mvolt) + \
              exp_conductance(20 * msecond, E= -60 * mvolt)
    print m
    print m[0](1. * mvolt, 2., 3.), m[1](1. * volt, 2., 3.), m[2](1. * volt, 2., 3.)



from brian import *
import numpy, os, pylab

"""
An implementation of a simple topographical network, like those used in Mehring 2005 or Yger 2011. 
Cells are aranged randomly on a 2D plane and connected according to a gaussian profile
P(r) = exp(-d**2/(2*sigma**2)), with delays depending linearly on the distances. 
Note that the exact number of synapses per neuron is not fixed here. 

To avoid any border conditions, the plane is considered to be toroidal. 
Script will generate an Synchronous Irregular (SI) slow regime with propagating
waves that will spread in various directions, wandering over the network. 

In addition, an external layer of Poisson sources will stimulates some cells on the network, with
a wiring scheme such that the word BRIAN will pop up. External rates can be turned off to observed the 
spontaneous activity of the 2D layer. One can observe that despite the inputs is constant, the network
is not always responding to it. 

The script will display, while running, the spikes and Vm of the excitatory cells. 

Varying sigma will show the various activity structures from a random network (s_lat > 1), to a very
locally connected one (s_lat < 0.1)

"""

### We are setting the global timestep of the simulation
Clock(0.1 * ms)

### Cell parameters ###
tau_m   = 20. * ms # Membrane time constant
c_m     = 0.2 * nF # Capacitance
tau_exc =  3. * ms # Synaptic time constant (excitatory)
tau_inh =  7. * ms # Synaptic time constant (inhibitory)
tau_ref =  5. * ms # Refractory period
El      = -80 * mV # Leak potential
Ee      =  0. * mV # Reversal potential (excitation)
Ei      = -70.* mV # Reversal potential (inhibition)
Vt      = -50 * mV # Spike Threhold 
Vr      = -60 * mV # Spike Reset

### Equation for a Conductance-based IAF ####
eqs = Equations('''
dv/dt  = (El-v)/tau_m + (ge*(Ee-v)+gi*(Ei-v))/c_m : volt
dge/dt = -ge*(1./tau_exc) : uS
dgi/dt = -gi*(1./tau_inh) : uS
''')

n_cells       = 12500                   # Total number of cells
n_exc         = int(0.8 * n_cells)      # 4:1 ratio for exc/inh
size          = 1.                      # Size of the network
simtime       = 1000 * ms               # Simulation time
sim_step      = 1 * ms                  # Display snapshots every sim_step ms
epsilon       = 0.02                    # Probability density
s_lat         = 0.2                     # Spread of the lateral connections
g_exc         = 4.  * nS                # Excitatory conductance
g_inh         = 64. * nS                # Inhibitory conductance
g_ext         = 200 * nS                # External drive
velocity      = 0.3 * mm/ms             # velocity
ext_rate      = 100 * Hz                # Rate of the external source
max_distance  = size * mm/numpy.sqrt(2) # Since this is a torus
max_delay     = max_distance/velocity   # Needed for the connectors

### Generate the images with the letters B, R, I, A, N
### To do that, we create a png image and read it as a matrix
pylab.figure()
pylab.text(0.125, 0.4, "B R I A N", size=80)
pylab.setp(gca(), xticks=[], yticks=[])
pylab.savefig("BRIAN.png")
brian_letters = imread("BRIAN.png")
os.remove("BRIAN.png")
brian_letters = numpy.flipud(mean(brian_letters,2)).T
pylab.close()

### We create the cells and generate random positons in [0, size]x[0, size]
all_cells          = NeuronGroup(n_cells, model=eqs, threshold=Vt, reset=Vr, refractory=tau_ref)
all_cells.position = size*numpy.random.rand(n_cells, 2)
exc_cells          = all_cells[0:n_exc]
inh_cells          = all_cells[n_exc:n_cells]

### We initialize v values slightly above Vt, to have initial spikes
all_cells.v        = El + 1.1*numpy.random.rand(n_cells) * (Vt - El)

### Now we create the source that will write the word BRIAN
sources            = PoissonGroup(1, ext_rate)
sources.position   = array([[0.5, 0.5]])

### Function to get the distance between one position and an array of positions
### This is needed to used the vectorized form of the connections in the brian.Connection objects
### Note that the plane is wrapped, to avoid any border effects.
def get_distance(x, y):
    d1       = abs(x - y)
    min_d    = numpy.minimum(d1, size - d1)
    return numpy.sqrt(numpy.sum(min_d**2, 1))

### Function returning the probabilities of connections as a functions of distances
def probas(i, j, x, y):
    distance = get_distance(x[i], y[j])
    return epsilon * numpy.exp(-distance**2/(2*s_lat**2)) 

### Function returning linear delays as function of distances
def delays(i, j, x, y):
    distance = get_distance(x[i], y[j])
    return 0.1*ms + (distance * mm )/ velocity

### Function assessing if a cell is located in a particular letter of the word BRIAN
### Return 0 if not, and 1 if yes.
def is_in_brian(i, j, x, y):
    a, b         = brian_letters.shape
    tmp_x, tmp_y = (y[j][:, 0]*a).astype(int), (y[j][:, 1]*b).astype(int)
    return 1 - brian_letters[tmp_x, tmp_y]


print "Building network with wrapped 2D gaussian profiles..." 
Ce = Connection(exc_cells, all_cells, 'ge', weight=g_exc, max_delay=max_delay, 
            sparseness=lambda i, j : probas(i, j, exc_cells.position, all_cells.position), 
            delay     =lambda i, j : delays(i, j, exc_cells.position, all_cells.position))
Ci = Connection(inh_cells, all_cells, 'gi', weight=g_inh, max_delay=max_delay, 
            sparseness=lambda i, j : probas(i, j, inh_cells.position, all_cells.position), 
            delay     =lambda i, j : delays(i, j, inh_cells.position, all_cells.position))
Cext = Connection(sources, all_cells, 'ge', weight=g_ext, max_delay=max_delay, 
            sparseness=lambda i, j : is_in_brian(i, j, sources.position, all_cells.position))

print "--> mean probability from excitatory synapses:", Ce.W.getnnz()/float(n_exc*n_cells) * 100, "%"
print "--> mean probability from inhibitory synapses:", Ci.W.getnnz()/float((n_cells - n_exc)*n_cells) * 100, "%"


print "Setting the recorders..." 
v_exc   = RecentStateMonitor(exc_cells, 'v', record=True)
s_exc   = SpikeCounter(exc_cells)


ion() # To enter the interactive mode
print "Initializing the plots..."
fig1    = pylab.subplot(211)
im      = fig1.scatter(all_cells.position[:n_exc, 0], all_cells.position[:n_exc, 1], c=[0]*n_exc)
im.set_clim(0, 1)
fig1.set_ylabel("spikes")
pylab.colorbar(im) 
fig2    = pylab.subplot(212) 
im      = fig2.scatter(all_cells.position[:n_exc, 0], all_cells.position[:n_exc, 1], c=[0]*n_exc)
im.set_clim(El, Vt)
fig2.set_ylabel("v")
pylab.colorbar(im) 


manager = pylab.get_current_fig_manager() 

print "Running network ..." 
for time in xrange(int((simtime/sim_step)/ms)):
    run(sim_step)    
    fig1.cla()
    fig1.set_title("t = %g s" %((sim_step * time)/ms))   
    idx     = s_exc.count > 0
    if numpy.sum(idx) > 0:
        im = fig1.scatter(all_cells.position[:n_exc, 0][idx], all_cells.position[:n_exc, 1][idx], c=[0]*n_exc)
    s_exc.count = numpy.zeros(n_exc) ## We reset the spike counter
    fig1.set_xlim(0, size)
    fig1.set_ylim(0, size)
    fig1.set_ylabel("spikes")
    im.set_clim(0, 1)
    setp(fig1, xticks=[], yticks=[])
    
    fig2.cla() 
    im = fig2.scatter(all_cells.position[:n_exc, 0], all_cells.position[:n_exc, 1], c=v_exc.values[-1])
    fig2.set_xlim(0, size)
    fig2.set_ylim(0, size)
    fig2.set_ylabel("v")
    im.set_clim(El, Vt)
    setp(fig2, xticks=[], yticks=[])

    manager.canvas.draw()
    manager.canvas.flush_events()
ioff() # To leave the interactive mode    
    



"""
Analysis of voltage traces.

Mainly about analysis of spike shapes.
"""
from numpy import *
from scipy import optimize
from scipy.signal import lfilter

__all__ = ['find_spike_criterion', 'spike_peaks', 'spike_onsets', 'find_onset_criterion',
         'slope_threshold', 'vm_threshold', 'spike_shape', 'spike_duration', 'reset_potential',
         'spike_mask', 'lowpass', 'spike_onsets_dv2', 'spike_onsets_dv3']

def lowpass(x, tau, dt=1.):
    """
    Low-pass filters x(t) with time constant tau.
    """
    a = exp(-dt / tau)
    return lfilter([1. - a], [1., -a], x)

def spike_duration(v, onsets=None, full=False):
    '''
    Average spike duration.
    Default: time from onset to next minimum.
    If full is True:
    * Time from onset to peak
    * Time from onset down to same value (spike width)
    * Total duration from onset to next minimum
    * Standard deviations for these 3 values
    '''
    if onsets is None: onsets = spike_onsets(v)
    dv = diff(v)
    total_duration = []
    time_to_peak = []
    spike_width = []
    for i, spike in enumerate(onsets):
        if i == len(onsets) - 1:
            next_spike = len(dv)
        else:
            next_spike = onsets[i + 1]
        total_duration.append(((dv[spike:next_spike - 1] <= 0) & (dv[spike + 1:next_spike] > 0)).argmax())
        time_to_peak.append((dv[spike:next_spike] <= 0).argmax())
        spike_width.append((v[spike + 1:next_spike] <= v[spike]).argmax())
    if full:
        return mean(time_to_peak), mean(spike_width), mean(total_duration), \
               std(time_to_peak), std(spike_width), std(total_duration)
    else:
        return mean(total_duration)

def reset_potential(v, peaks=None, full=False):
    '''
    Average reset potential, calculated as next minimum after spike peak.
    If full is True, also returns the standard deviation.
    '''
    if peaks is None: peaks = spike_peaks(v)
    dv = diff(v)
    reset = []
    for i, spike in enumerate(peaks):
        if i == len(peaks) - 1:
            next_spike = len(dv)
        else:
            next_spike = peaks[i + 1]
        reset.append(v[spike + ((dv[spike:next_spike - 1] <= 0) & (dv[spike + 1:next_spike] > 0)).argmax() + 1])
    if full:
        return mean(reset), std(reset)
    else:
        return mean(reset)

def find_spike_criterion(v):
    '''
    This is a rather complex method to determine above which voltage vc
    one should consider that a spike is produced.
    We look in phase space (v,dv/dt), at the horizontal axis dv/dt=0.
    We look for a voltage for which the voltage cannot be still.
    Algorithm: find the largest interval of voltages for which there is no
    sign change of dv/dt, and pick the middle.
    '''
    # Rather: find the lowest maximum?
    dv = diff(v)
    sign_changes = ((dv[1:] * dv[:-1]) <= 0).nonzero()[0]
    vc = v[sign_changes + 1]
    i = argmax(diff(vc))
    return .5 * (vc[i] + vc[i + 1])

def spike_peaks(v, vc=None):
    '''
    Returns the indexes of spike peaks.
    vc is the spike criterion (voltage above which we consider we have a spike)
    '''
    # Possibly: add refractory criterion
    vc = vc or find_spike_criterion(v)
    dv = diff(v)
    spikes = ((v[1:] > vc) & (v[:-1] < vc)).nonzero()[0]
    peaks = []
    if len(spikes) > 0:
        for i in range(len(spikes) - 1):
            peaks.append(spikes[i] + (dv[spikes[i]:spikes[i + 1]] <= 0).nonzero()[0][0])
        decreasing = (dv[spikes[-1]:] <= 0).nonzero()[0]
        if len(decreasing) > 0:
            peaks.append(spikes[-1] + decreasing[0])
        else:
            peaks.append(len(dv)) # last element (maybe should be deleted?)
    return array(peaks)

def spike_onsets(v, criterion=None, vc=None):
    '''
    Returns the indexes of spike onsets.
    vc is the spike criterion (voltage above which we consider we have a spike).
    First derivative criterion (dv>criterion).
    '''
    vc = vc or find_spike_criterion(v)
    criterion = criterion or find_onset_criterion(v, vc=vc)
    peaks = spike_peaks(v, vc)
    dv = diff(v)
    d2v = diff(dv)
    previous_i = 0
    j = 0
    l = []

    for i in peaks:
        # Find last peak of derivative (commented: point where derivative is largest)
        # inflexion = previous_i + argmax(dv[previous_i:i])
        inflexion=where(d2v[previous_i:i-2]*d2v[previous_i+1:i-1]<0)[0][-1]+2+previous_i
        j += max((dv[j:inflexion] < criterion).nonzero()[0]) + 1
        l.append(j)
        previous_i = i

    return array(l)

def spike_onsets_dv2(v, vc=None):
    '''
    Returns the indexes of spike onsets.
    vc is the spike criterion (voltage above which we consider we have a spike).
    Maximum of 2nd derivative.
    DOESN'T SEEM GOOD
    '''
    vc = vc or find_spike_criterion(v)
    peaks = spike_peaks(v, vc)
    d2v = diff(diff(v))
    d3v = diff(d2v) # I'm guessing you have to shift v by 1/2 per differentiation
    j = 0
    l = []
    previous_i=0
    for i in peaks:
        # Find peak of derivative
        inflexion=where(d2v[previous_i:i-1]*d2v[previous_i+1:i]<0)[0][-1]+2+previous_i
        j += max(((d3v[j:inflexion - 1] > 0) & (d3v[j + 1:inflexion] < 0)).nonzero()[0]) # +2?
        l.append(j)
        previous_i=i
    return array(l)

def spike_onsets_dv3(v, vc=None):
    '''
    Returns the indexes of spike onsets.
    vc is the spike criterion (voltage above which we consider we have a spike).
    Maximum of 3rd derivative.
    DOESN'T SEEM GOOD
    '''
    vc = vc or find_spike_criterion(v)
    peaks = spike_peaks(v, vc)
    dv4 = diff(diff(diff(diff(v))))
    j = 0
    l = []
    for i in peaks:
        # Find peak of derivative (alternatively: last sign change of d2v, i.e. last local peak)
        j += max(((dv4[j:i - 1] > 0) & (dv4[j + 1:i] < 0)).nonzero()[0]) + 3
        l.append(j)
    return array(l)

def find_onset_criterion(v, guess=0.0001, vc=None):
    '''
    Finds the best criterion on dv/dt to determine spike onsets,
    based on minimum threshold variability.
    '''
    vc = vc or find_spike_criterion(v)
    return float(optimize.fmin(lambda x:std(v[spike_onsets(v, x, vc)]), guess, disp=0))

def spike_shape(v, onsets=None, before=100, after=100):
    '''
    Spike shape (before peaks). Aligned on spike onset by default
    (to align on peaks, just pass onsets=peaks).
    
    onsets: spike onset times
    before: number of timesteps before onset
    after: number of timesteps after onset
    '''
    if onsets is None: onsets = spike_onsets(v)
    shape = zeros(after + before)
    for i in onsets:
        v0 = v[max(0, i - before):i + after]
        shape[len(shape) - len(v0):] += v0
    return shape / len(onsets)

def vm_threshold(v, onsets=None, T=None):
    '''
    Average membrane potential before spike threshold (T steps).
    '''
    if onsets is None: onsets = spike_onsets(v)
    l = []
    for i in onsets:
        l.append(mean(v[max(0, i - T):i]))
    return array(l)

def slope_threshold(v, onsets=None, T=None):
    '''
    Slope of membrane potential before spike threshold (T steps).
    Returns all slopes as an array.
    '''
    if onsets is None: onsets = spike_onsets(v)
    l = []
    for i in onsets:
        v0 = v[max(0, i - T):i]
        M = len(v0)
        x = arange(M) - M + 1
        slope = sum((v0 - v[i]) * x) / sum(x ** 2)
        l.append(slope)
    return array(l)

def spike_mask(v, spikes=None, T=None):
    '''
    Returns an array of booleans which are True in spikes.
    spikes: starting points of spikes (default: onsets)
    T: duration (default: next minimum)
    
    Ex:
      v=v[spike_mask(v)] # only spikes
      v=v[-spike_mask(v)] # subthreshold trace
    '''
    if spikes is None: spikes = spike_onsets(v)
    ind = (v == 1e9)
    if T is None:
        dv = diff(v)
        for i, spike in enumerate(spikes):
            if i == len(spikes) - 1: next_spike = len(dv)
            else: next_spike = spikes[i + 1]
            T = ((dv[spike:next_spike - 1] <= 0) & (dv[spike + 1:next_spike] > 0)).argmax()
            ind[spike:spike + T + 1] = True
    else: # fixed duration
        for i in spikes:
            ind[i:i + T] = True
    return ind



'''
A pure Python version of the CUBA example, that reproduces basic Brian principles.

Fixed overhead is about 0.35 s.
Real time simulation for N=1300.
'''
from pylab import *
import bisect
from time import time
from random import sample
from scipy import random as scirandom

"""
Parameters
"""
N = 1300        # number of neurons
Ne = int(N * 0.8) # excitatory neurons 
Ni = N - Ne       # inhibitory neurons
mV = ms = 1e-3    # units
dt = 0.1 * ms     # timestep
taum = 20 * ms    # membrane time constant
taue = 5 * ms
taui = 10 * ms
p = 80.0 / N # connection probability (80 synapses per neuron)
Vt = -1 * mV      # threshold = -50+49
Vr = -11 * mV     # reset = -60+49
we = 60 * 0.27 / 10 # excitatory weight
wi = -20 * 4.5 / 10 # inhibitory weight
duration = 1000 * ms

"""
Equations
---------
eqs='''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

This is a linear system, so each update corresponds to
multiplying the state matrix by a (3,3) 'update matrix'
"""

# Update matrix
A = array([[exp(-dt / taum), 0, 0],
         [taue / (taum - taue) * (exp(-dt / taum) - exp(-dt / taue)), exp(-dt / taue), 0],
         [taui / (taum - taui) * (exp(-dt / taum) - exp(-dt / taui)), 0, exp(-dt / taui)]]).T

"""
State variables
---------------
P=NeuronGroup(4000,model=eqs,
              threshold=-50*mV,reset=-60*mV)
"""
S = zeros((3, N))

"""
Initialisation
--------------
P.v=-60*mV+10*mV*rand(len(P))
"""
S[0, :] = rand(N) * (Vt - Vr) + Vr # Potential: uniform between reset and threshold

"""
Connectivity matrices
---------------------
Pe=P.subgroup(3200) # excitatory group
Pi=P.subgroup(800)  # inhibitory group
Ce=Connection(Pe,P,'ge',weight=1.62*mV,sparseness=p)
Ci=Connection(Pi,P,'gi',weight=-9*mV,sparseness=p)
"""
We_target = []
We_weight = []
for _ in range(Ne):
    k = scirandom.binomial(N, p, 1)[0]
    target = sample(xrange(N), k)
    target.sort()
    We_target.append(array(target))
    We_weight.append(array([1.62 * mV] * k))
Wi_target = []
Wi_weight = []
for _ in range(Ni):
    k = scirandom.binomial(N, p, 1)[0]
    target = sample(xrange(N), k)
    target.sort()
    Wi_target.append(array(target))
    Wi_weight.append(array([-9 * mV] * k))

"""
Spike monitor
-------------
M=SpikeMonitor(P)

will contain a list of (i,t), where neuron i spiked at time t.
"""
spike_monitor = [] # Empty list of spikes

"""
State monitor
-------------
trace=StateMonitor(P,'v',record=0) # record only neuron 0
"""
trace = [] # Will contain v(t) for each t (for neuron 0)

"""
Simulation
----------
run(duration)
"""
t1 = time()
t = 0 * ms
while t < duration:
    # STATE UPDATES
    S[:] = dot(A, S)

    # Threshold
    all_spikes = (S[0, :] > Vt).nonzero()[0]     # List of neurons that meet threshold condition

    # PROPAGATION OF SPIKES
    # Excitatory neurons
    spikes = all_spikes[0:bisect.bisect_left(all_spikes, Ne)]
    for i in spikes:
        S[1, We_target[i]] += We_weight[i]

    # Inhibitory neurons
    spikes = all_spikes[bisect.bisect_left(all_spikes, Ne):] - Ne
    for i in spikes:
        S[2, Wi_target[i]] += Wi_weight[i]

    # Reset neurons after spiking
    S[0, all_spikes] = Vr                       # Reset membrane potential

    # Spike monitor
    spike_monitor += [(i, t) for i in all_spikes]

    # State monitor
    trace.append(S[0, 0])

    t += dt

t2 = time()
print "Simulated in", t2 - t1, "s"
print len(spike_monitor), "spikes"

"""
Plot
----
subplot(211)
raster_plot(M)
subplot(212)
plot(trace.times/ms,trace[0]/mV)
show()
"""
subplot(211)
i, t = zip(*spike_monitor)
plot(i, t, '.')
subplot(212)
plot(arange(len(trace)) * dt / ms, array(trace) / mV)
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Brian
"""

__docformat__ = "restructuredtext en"

import warnings as _warnings
from scipy import *
try:
    from pylab import *
except:
    _warnings.warn("Couldn't import pylab.")
if 'x' in globals(): del x # for some reason x is defined as 'symlog' by pylab!
if 'f' in globals(): del f

from clock import *
from connections import *
from directcontrol import *
from stateupdater import *
from monitor import *
from network import *
from neurongroup import *
from plotting import *
from reset import *
from threshold import *
from units import *
from tools import *
from equations import *
from globalprefs import *
from unitsafefunctions import *
from stdunits import *
from membrane_equations import *
from compartments import *
from log import *
from magic import *
from stdp import *
from stp import *
from timedarray import *
from tests.simpletest import *

__version__ = '1.3.1'
__release_date__ = '2011-12-16'

### Define global preferences which are not defined anywhere else

define_global_preference(
    'useweave', 'False',
    desc="""
         Defines whether or not functions should use inlined compiled
         C code where defined. Requires a compatible C++ compiler.
         The ``gcc`` and ``g++`` compilers are probably the easiest
         option (use Cygwin on Windows machines). See also the
         ``weavecompiler`` global preference.
         """)
set_global_preferences(useweave=False)
define_global_preference(
    'weavecompiler', 'gcc',
    desc='''
         Defines the compiler to use for weave compilation. On Windows machines, installing
         Cygwin is the easiest way to get access to the gcc compiler.
         ''')
set_global_preferences(weavecompiler='gcc')

define_global_preference(
    'gcc_options', "['-ffast-math']",
    desc='''
         Defines the compiler switches passed to the gcc compiler. For gcc versions
         4.2+ we recommend using ``-march=native``. By default, the ``-ffast-math``
         optimisations are turned on - if you need IEEE guaranteed results, turn
         this switch off.
         ''')
set_global_preferences(gcc_options=['-ffast-math'])

define_global_preference(
    'usecodegen', 'False',
    desc='''
         Whether or not to use experimental code generation support.
         ''')
set_global_preferences(usecodegen=False)
define_global_preference(
    'usecodegenweave', 'False',
    desc='''
         Whether or not to use C with experimental code generation support.
         ''')
set_global_preferences(usecodegenweave=False)
define_global_preference(
    'usecodegenstateupdate', 'True',
    desc='''
         Whether or not to use experimental code generation support on state updaters.
         ''')
set_global_preferences(usecodegenstateupdate=True)
define_global_preference(
    'usecodegenreset', 'False',
    desc='''
         Whether or not to use experimental code generation support on resets.
         Typically slower due to weave overheads, so usually leave this off.
         ''')
set_global_preferences(usecodegenreset=False)
define_global_preference(
    'usecodegenthreshold', 'True',
    desc='''
         Whether or not to use experimental code generation support on thresholds.
         ''')
set_global_preferences(usecodegenthreshold=True)

define_global_preference(
    'usenewpropagate', 'False',
    desc='''
         Whether or not to use experimental new C propagation functions.
         ''')
set_global_preferences(usenewpropagate=False)

define_global_preference(
    'usecstdp', 'False',
    desc='''
         Whether or not to use experimental new C STDP.
         ''')
set_global_preferences(usecstdp=False)

define_global_preference(
    'brianhears_usegpu', 'False',
    desc='''
         Whether or not to use the GPU (if available) in Brian.hears. Support
         is experimental at the moment, and requires the PyCUDA package to be
         installed.
         ''')
set_global_preferences(brianhears_usegpu=False)

# check if we were run from a file or some other source, and set the default
# behaviour for magic functions accordingly
import inspect as _inspect
import os as _os
_of = _inspect.getouterframes(_inspect.currentframe())
if len(_of) > 1 and _os.path.exists(_of[1][1]):
    _magic_useframes = True
else:
    _magic_useframes = False
define_global_preference(
    'magic_useframes', str(_magic_useframes),
    desc="""
         Defines whether or not the magic functions should search
         for objects defined only in the calling frame or if they
         should find all objects defined in any frame. This should
         be set to ``False`` if you are using Brian from an interactive
         shell like IDLE or IPython where each command has its own
         frame, otherwise set it to ``True``.
         """)
set_global_preferences(magic_useframes=_magic_useframes)

### Update documentation for global preferences
import globalprefs as _gp
_gp.__doc__ += _gp.globalprefdocs

try:
    import brian_global_config
except ImportError:
    pass

try:
    import nose

    @nose.tools.nottest
    def run_all_tests():
        import tests
        tests.go()

except ImportError:

    def run_all_tests():
        print "Brian test framework requires 'nose' package."



'''
Test of Shinomoto's automatic bin size selection
Probably not working yet!
'''
from brian import *
import time

start_time = time.time()
taum = 20 * ms
taue = 5 * ms
taui = 10 * ms
Vt = -50 * mV
Vr = -60 * mV
El = -49 * mV

eqs = Equations('''
dv/dt  = (ge+gi-(v-El))/taum : volt
dge/dt = -ge/taue : volt
dgi/dt = -gi/taui : volt
''')

P = NeuronGroup(4000, model=eqs, threshold=Vt, reset=Vr, refractory=5 * ms)
P.v = Vr
P.ge = 0 * mV
P.gi = 0 * mV

Pe = P.subgroup(3200)
Pi = P.subgroup(800)
we = (60 * 0.27 / 10) * mV # excitatory synaptic weight (voltage)
wi = (-20 * 4.5 / 10) * mV # inhibitory synaptic weight
Ce = Connection(Pe, P, 'ge', weight=we, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight=wi, sparseness=0.02)
P.v = Vr + rand(len(P)) * (Vt - Vr)

# Record the number of spikes
Me = PopulationSpikeCounter(Pe)
Mi = PopulationSpikeCounter(Pi)
# A population rate monitor
M = PopulationRateMonitor(P[:200])

print "Network construction time:", time.time() - start_time, "seconds"
print len(P), "neurons in the network"
print "Simulation running..."
run(1 * msecond)
start_time = time.time()

run(1 * second)

duration = time.time() - start_time
print "Simulation time:", duration, "seconds"
print Me.nspikes, "excitatory spikes"
print Mi.nspikes, "inhibitory spikes"
plot(M.times / ms, M.smooth_rate(filter='flat'))
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Compartmental models for Brian.
'''
from equations import *
from membrane_equations import *
from units import check_units, ohm

#TODO: add MembraneEquation
class Compartments(Equations):
    """
    Creates a compartmental model from a dictionary
    of MembraneEquation objects. Compartments are initially unconnected.
    Caution: the original objects are modified.
    """
    def __init__(self, comps):
        Equations.__init__(self, '')
        for name, eqs in comps.iteritems():
            name = str(name)
            # Change variable names
            vars = eqs._units.keys()
            for var in vars:
                eqs.substitute(var, var + '_' + name)
            self += eqs

    @check_units(Ra=ohm)
    def connect(self, a, b, Ra):
        """
        Connects compartment a to compartment b with axial resistance Ra.
        """
        a, b = str(a), str(b)
        # Axial current from a to b
        Ia_name = 'Ia_' + a + '_' + b
        self += Equations('Ia=(va-vb)*invRa : amp', Ia=Ia_name, invRa=1. / Ra, va='vm_' + a, vb='vm_' + b)
        # Add the current to both compartments
        self._string['__membrane_Im_' + a] += '-' + Ia_name
        self._string['__membrane_Im_' + b] += '+' + Ia_name



# TODO: update all of this with the new interface/buffering mechanism
# TODO: decide on a good interface for online sounds, that is general

from brian import *
from numpy import *
import numpy
import wave
import array as pyarray
import time
try:
    import pygame
    have_pygame = True
except ImportError:
    have_pygame = False
try:
    from scikits.samplerate import resample
    have_scikits_samplerate = True
except (ImportError, ValueError):
    have_scikits_samplerate = False
from bufferable import Bufferable
from sounds import Sound, BaseSound

__all__ = ['OnlineSound',
                'OnlineWhiteNoise', 'OnlineWhiteNoiseBuffered',
                'OnlineWhiteNoiseShifted',
           ]

class OnlineSound(BaseSound):
    def __init__(self): 
        pass
        
    def update(self):
        pass
    
class OnlineWhiteNoise(OnlineSound):
    '''
    Noise generator which produces one sample at a time online
    input parameters are the mean mu and the variance sigma
    default mu=0, sigma=1
    '''
    def __init__(self,mu=None,sigma=None,tomux=1): 
        if mu==None:
            self.mu=0
        if sigma==None:
            self.sigma=1
        self.tomux=tomux
        self.nchannels = 1

    def buffer_init(self):
        pass
        
    def buffer_fetch(self, start, end):
        samples = end-start
        temp = (self.mu+sqrt(self.sigma)*randn(samples))*self.tomux
        temp = temp.reshape(len(temp),1)
        return temp



class OnlineWhiteNoiseBuffered(OnlineSound):
    def __init__(self,samplerate,mu,sigma,max_abs_itd): 
        self.samplerate=samplerate
        self.length_buffer=int(max_abs_itd * self.samplerate)
        
        self.mu=mu
        self.sigma=sigma
        self.buffer=[0]*(2*self.length_buffer+1)
        
    def update(self):
        self.buffer.pop()
        self.buffer.insert(0,self.mu+self.sigma*randn(1))
        return self.buffer[self.length_buffer]
    
class OnlineWhiteNoiseShifted(OnlineSound):
    def __init__(self,samplerate,online_white_noise_buffered,shift=lambda:randn(1)*ms,time_interval=-1*ms): 
        #self.shift_applied=[] 
        self.samplerate=samplerate
        self.interval_in_sample= int(time_interval *self.samplerate)
        #print self.interval_in_sample 
        self.count=0
        self.shift=shift
        self.length_buffer=online_white_noise_buffered.length_buffer
        self.shift_in_sample=int(shift()* self.samplerate)
        if abs(self.shift_in_sample) > self.length_buffer:
            self.shift_in_sample=sign(self.shift_in_sample)*self.length_buffer
        self.ITDused=[]
        self.ITDused.append(self.shift_in_sample/self.samplerate)
        self.reference=online_white_noise_buffered
        
        
    def update(self):
        if self.count ==self.interval_in_sample:
            self.shift_in_sample=int(self.shift()* self.samplerate)
            if abs(self.shift_in_sample) > self.length_buffer:
                self.shift_in_sample=sign(self.shift_in_sample)*self.length_buffer
            self.ITDused.append(self.shift_in_sample/self.samplerate)
            #print self.shift()
            self.count=0
            
        self.count=self.count+1   
#        print self.length_buffer
#        print self.shift_in_sample
#        print self.length_buffer+1+self.shift_in_sample
        return self.reference.buffer[self.length_buffer+self.shift_in_sample]



'''
Example with short term plasticity model
Neurons with regular inputs and depressing synapses
'''
from brian import *
from brian.experimental.connectionmonitor import *
import pygame

tau_e=3*ms
taum=10*ms
A_SE=250*pA
Rm=100*Mohm
N=10

eqs='''
dx/dt=rate : 1
rate : Hz
'''

input=NeuronGroup(N,model=eqs,threshold=1.,reset=0)
input.rate=linspace(5*Hz,30*Hz,N)

eqs_neuron='''
dv/dt=(Rm*i-v)/taum:volt
di/dt=-i/tau_e:amp
'''
neuron=NeuronGroup(N,model=eqs_neuron)

C=Connection(input,neuron,'i')
C.connect_one_to_one(weight=A_SE)
stp=STP(C,taud=1*ms,tauf=100*ms,U=.1) # facilitation
#stp=STP(C,taud=100*ms,tauf=10*ms,U=.6) # depression
trace=StateMonitor(neuron,'v',record=[0,N-1])

M=ConnectionMonitor(C, display='pygame', size=(640, 640))

run(1000*ms)
pygame.quit()
subplot(211)
plot(trace.times/ms,trace[0]/mV)
title('Vm')
subplot(212)
plot(trace.times/ms,trace[N-1]/mV)
title('Vm')
show()




from brian import *
set_global_preferences(usenewbrianhears=True,
                       useweave=False)
from brian.hears import *

x = Sound(array([randn(1000), zeros(1000)]).T, samplerate=44100*Hz)
w = Sound(array([linspace(-1, 1, 1000), sin(linspace(0, 2*pi, 1000))]).T, samplerate=44100*Hz)
#w = Sound(sin(linspace(0, 2*pi, 1000)), samplerate=44100*Hz)

fb = RestructureFilterbank((x, w), 2, 'serial', 3)
#fb = RestructureFilterbank((x, w), 2, 'interleave', 3)
#fb = RestructureFilterbank((x, w), indexmapping=[0, 2, 3, 1])

fb.buffer_init()
y = fb.buffer_fetch(0, 1000)

subplot(211)
plot(x)
plot(w)
subplot(212)
for i, z in enumerate(y.T):
    plot(z+5*i)
show()



'''
Neuronal morphology module for Brian.

TODO:
* Maybe str: should give a text representation of the tree
* set_length etc.: should be recursive
* check_consistency: check that lengths, area and coordinates are consistent (and fix)
* rescale: change number of compartments
'''
from brian.group import Group
from scipy import rand
from numpy import *
from brian.units import meter
from brian.stdunits import um
import warnings
from pylab import figure
import copy
try:
    from mpl_toolkits.mplot3d import Axes3D
except:
    warnings.warn('Pylab 0.99.1 is required for 3D plots')

__all__ = ['Morphology', 'Cylinder', 'Soma']


class Morphology(object):
    '''
    Neuronal morphology (=tree of branches).
    '''
    def __init__(self, filename=None, n=None):
        self.children = []
        self._namedkid = {}
        self.iscompressed = False
        if filename is not None:
            self.loadswc(filename)
        elif n is not None: # Creates a branch with n compartments
            # The problem here is that these parameters should have some self-consistency
            self.x, self.y, self.z, self.diameter, self.length, self.area = [zeros(n) for _ in range(6)]

    def set_length(self):
        '''
        Sets the length of compartments according to their coordinates
        '''
        x = hstack((0 * um, self.x))
        y = hstack((0 * um, self.y))
        z = hstack((0 * um, self.z))
        self.length = sum((x[1:] - x[:-1]) ** 2 + (y[1:] - y[:-1]) ** 2 + (z[1:] - z[:-1]) ** 2) ** .5

    def set_area(self):
        '''
        Sets the area of compartments according to diameter and length (assuming cylinders)
        '''
        self.area = pi * self.diameter * self.length

    def set_coordinates(self):
        '''
        Sets the coordinates of compartments according to their lengths (taking a random direction)
        '''
        l = cumsum(self.length)
        theta = rand()*2 * pi
        phi = rand()*2 * pi
        self.x = l * sin(theta) * cos(phi)
        self.y = l * sin(theta) * sin(phi)
        self.z = l * cos(theta)

    def loadswc(self, filename):
        '''
        Reads a SWC file containing a neuronal morphology.
        Large database at http://neuromorpho.org/neuroMorpho        
        Information below from http://www.mssm.edu/cnic/swc.html
        
        SWC File Format
        
        The format of an SWC file is fairly simple. It is a text file consisting of a header with various fields beginning with a # character, and a series of three dimensional points containing an index, radius, type, and connectivity information. The lines in the text file representing points have the following layout. 
        n T x y z R P
        n is an integer label that identifies the current point and increments by one from one line to the next.
        T is an integer representing the type of neuronal segment, such as soma, axon, apical dendrite, etc. The standard accepted integer values are given below.
            * 0 = undefined
            * 1 = soma
            * 2 = axon
            * 3 = dendrite
            * 4 = apical dendrite
            * 5 = fork point
            * 6 = end point
            * 7 = custom
        x, y, z gives the cartesian coordinates of each node.
        R is the radius at that node.
        P indicates the parent (the integer label) of the current point or -1 to indicate an origin (soma). 
        '''
        # 1) Create the list of segments, each segment has a list of children
        lines = open(filename).read().splitlines()
        segment = [] # list of segments
        types = ['undefined', 'soma', 'axon', 'dendrite', 'apical', 'fork', 'end', 'custom']
        previousn = -1
        for line in lines:
            if line[0] != '#': # comment
                numbers = line.split()
                n = int(numbers[0]) - 1
                T = types[int(numbers[1])]
                x = float(numbers[2]) * um
                y = float(numbers[3]) * um
                z = float(numbers[4]) * um
                R = float(numbers[5]) * um
                P = int(numbers[6]) - 1 # 0-based indexing
                if (n != previousn + 1):
                    raise ValueError, "Bad format in file " + filename
                seg = dict(x=x, y=y, z=z, T=T, diameter=2 * R, parent=P, children=[])
                location = (x, y, z)
                if T == 'soma':
                    seg['area'] = 4 * pi * R ** 2
                    seg['length'] = 0 * um
                else: # dendrite
                    locationP = (segment[P]['x'], segment[P]['y'], segment[P]['z'])
                    seg['length'] = (sum((array(location) - array(locationP)) ** 2)) ** .5 * meter
                    seg['area'] = seg['length'] * 2 * pi * R
                if P >= 0:
                    segment[P]['children'].append(n)
                segment.append(seg)
                previousn = n
        # We assume that the first segment is the root
        self.create_from_segments(segment)

    def create_from_segments(self, segment, origin=0):
        """
        Recursively create the morphology from a list of segments.
        Each segment has attributes: x,y,z,diameter,area,length (vectors) and children (list).
        It also creates a dictionary of names (_namedkid).
        """
        n = origin
        if segment[origin]['T'] != 'soma': # if it's a soma, only one compartment
            while (len(segment[n]['children']) == 1) and (segment[n]['T'] != 'soma'): # Go to the end of the branch
                n += 1
        # End of branch
        branch = segment[origin:n + 1]
        # Set attributes
        self.diameter, self.length, self.area, self.x, self.y, self.z = \
            zip(*[(seg['diameter'], seg['length'], seg['area'], seg['x'], seg['y'], seg['z']) for seg in branch])
        self.diameter, self.length, self.area, self.x, self.y, self.z = array(self.diameter), array(self.length), \
            array(self.area), array(self.x), array(self.y), array(self.z)
        self.type = segment[n]['T'] # normally same type for all compartments in the branch
        # Create children (list)
        self.children = [Morphology().create_from_segments(segment, origin=c) for c in segment[n]['children']]
        # Create dictionary of names (enumerates children from number 1)
        for i, child in enumerate(self.children):
            self._namedkid[str(i + 1)] = child
            # Name the child if possible
            if child.type in ['soma', 'axon', 'dendrite']:
                if child.type in self._namedkid:
                    self._namedkid[child.type] = None # two children with the same name: erase (see next block)
                else:
                    self._namedkid[child.type] = child
        # Erase useless names
        for k in self._namedkid.keys():
            if self._namedkid[k] is None:
                del self._namedkid[k]
        # If two kids, name them L (left) and R (right)
        if len(self.children) == 2:
            self._namedkid['L'] = self._namedkid['1']
            self._namedkid['R'] = self._namedkid['2']
        return self

    def branch(self):
        '''
        Returns the current branch without the children.
        '''
        morpho = copy.copy(self)
        morpho.children = []
        morpho._namedkid = {}
        return morpho

    def __getitem__(self, x):
        """
        Returns the subtree named x.
        Ex.: neuron['axon'] or neuron['11213']
        neuron[10*um:20*um] returns the subbranch from 10 um to 20 um.
        neuron[10*um] returns one compartment.
        neuron[5] returns compartment number 5.
        
        TODO:
        neuron[:] returns the full branch.
        """
        if type(x) == type(0): # int: returns one compartment
            morpho = self.branch()
            i = x
            j = i + 1
            morpho.diameter = morpho.diameter[i:j]
            morpho.length = morpho.length[i:j]
            morpho.area = morpho.area[i:j]
            morpho.x = morpho.x[i:j]
            morpho.y = morpho.y[i:j]
            morpho.z = morpho.z[i:j]
            if hasattr(morpho, '_origin'):
                morpho._origin += i
            return morpho
        elif isinstance(x, slice): # neuron[10*um:20*um]
            morpho = self.branch()
            start, stop = x.start, x.stop
            l = cumsum(morpho.length) # coordinate on the branch
            i = searchsorted(l, start)
            j = searchsorted(l, stop)
            morpho.diameter = morpho.diameter[i:j]
            morpho.length = morpho.length[i:j]
            morpho.area = morpho.area[i:j]
            morpho.x = morpho.x[i:j]
            morpho.y = morpho.y[i:j]
            morpho.z = morpho.z[i:j]
            if hasattr(morpho, '_origin'):
                morpho._origin += i
            return morpho
        elif isinstance(x, float): # neuron[10*um]
            morpho = self.branch()
            l = cumsum(morpho.length) # coordinate on the branch
            i = searchsorted(l, x)
            j = i + 1
            morpho.diameter = morpho.diameter[i:j]
            morpho.length = morpho.length[i:j]
            morpho.area = morpho.area[i:j]
            morpho.x = morpho.x[i:j]
            morpho.y = morpho.y[i:j]
            morpho.z = morpho.z[i:j]
            if hasattr(morpho, '_origin'):
                morpho._origin += i
            return morpho
        else:
            x = str(x) # convert int to string
            if (len(x) > 1) and all([c in 'LR123456789' for c in x]): # binary string of the form LLLRLR or 1213 (or mixed)
                return self._namedkid[x[0]][x[1:]]
            elif x in self._namedkid:
                return self._namedkid[x]
            else:
                raise AttributeError, "The subtree " + x + " does not exist"

    def __setitem__(self, x, kid):
        """
        Inserts the subtree and name it x.
        Ex.: neuron['axon'] or neuron['11213']
        If the tree already exists with another name, then it creates a synonym
        for this tree.
        The coordinates of the subtree are relative before function call,
        and are absolute after function call.
        """
        x = str(x) # convert int to string
        if (len(x) > 1) and all([c in 'LR123456789' for c in x]): # binary string of the form LLLRLR or 1213 (or mixed)
            self._namedkid[x[0]][x[1:]] = kid
        elif x in self._namedkid:
            raise AttributeError, "The subtree " + x + " already exists"
        else:
            # Update coordinates
            kid.x += self.x[-1]
            kid.y += self.y[-1]
            kid.z += self.z[-1]
            if kid not in self.children:
                self.children.append(kid)
                self._namedkid[str(len(self.children))] = kid # numbered child
            self._namedkid[x] = kid

    def __delitem__(self, x):
        """
        Removes the subtree x.
        """
        x = str(x) # convert int to string
        if (len(x) > 1) and all([c in 'LR123456789' for c in x]): # binary string of the form LLLRLR or 1213 (or mixed)
            del self._namedkid[x[0]][x[1:]]
        elif x in self._namedkid:
            child = self._namedkid[x]
            # Delete from name dictionary
            for name, kid in self._namedkid.items():
                if kid is child: del self._namedkid[name]
            # Delete from list of children
            for i, kid in enumerate(self.children):
                if kid is child: del self.children[i]
        else:
            raise AttributeError, "The subtree " + x + " does not exist"

    def __getattr__(self, x):
        """
        Returns the subtree named x.
        Ex.: axon=neuron.axon
        """
        return self[x]

    def __setattr__(self, x, kid):
        """
        Attach a subtree and named it x. If the subtree is None then the
        subtree x is deleted.
        Ex.: neuron.axon=Soma(diameter=10*um)
        Ex.: neuron.axon=None
        """
        if isinstance(kid, Morphology):
            if kid is None:
                del self[x]
            else:
                self[x] = kid
        else: # If it is not a subtree, then it's a normal class attribute
            object.__setattr__(self, x, kid)

    def __len__(self):
        """
        Returns the total number of compartments.
        """
        return len(self.x) + sum(len(child) for child in self.children)

    def compress(self, diameter=None, length=None, area=None, x=None, y=None, z=None, origin=0):
        """
        Compresses the tree by changing the compartment vectors to views on
        a matrix (or vectors). The morphology cannot be changed anymore but
        all other functions should work normally.
        
        origin : offset in the base matrix
        """
        self._origin = origin
        n = len(self.x)
        # Update values of vectors
        diameter[:n] = self.diameter
        length[:n] = self.length
        area[:n] = self.area
        x[:n] = self.x
        y[:n] = self.y
        z[:n] = self.z
        # Attributes are now views on these vectors
        self.diameter = diameter[:n]
        self.length = length[:n]
        self.area = area[:n]
        self.x = x[:n]
        self.y = y[:n]
        self.z = z[:n]
        for kid in self.children:
            kid.compress(diameter=diameter[n:], length=length[n:], area=area[n:], x=x[n:], y=y[n:], z=z[n:], origin=n)
            n += len(kid)
        self.iscompressed = True

    def plot(self, axes=None, simple=True, origin=None):
        """
        Plots the morphology in 3D. Units are um.
        axes : the figure axes (new figure if not given)
        simple : if True, the diameter of branches is ignored
        """
        if axes is None: # new figure
            fig = figure()
            axes = Axes3D(fig)
        x, y, z, d = self.x / um, self.y / um, self.z / um, self.diameter / um
        if origin is not None:
            x0, y0, z0 = origin
            x = hstack((x0, x))
            y = hstack((y0, y))
            z = hstack((z0, z))
        if len(x) == 1: # root with a single compartment: probably just the soma
            axes.plot(x, y, z, "r.", linewidth=d[0])
        else:
            if simple:
                axes.plot(x, y, z, "k")
            else: # linewidth reflects compartment diameter
                for n in range(1, len(x)):
                    axes.plot([x[n - 1], x[n]], [y[n - 1], y[n]], [z[n - 1], z[n]], 'k', linewidth=d[n - 1])
        for c in self.children:
            c.plot(origin=(x[-1], y[-1], z[-1]), axes=axes, simple=simple)


class Cylinder(Morphology):
    """
    A cylinder.
    """
    def __init__(self, length=None, diameter=None, n=1, type=None, x=None, y=None, z=None):
        """
        Creates a cylinder.
        n: number of compartments.
        type : 'soma', 'axon' or 'dendrite'
        x,y,z : end point (relative to origin of cylinder)
        length is optional (and ignored) if x,y,z is specified
        If x,y,z unspecified: random direction
        """
        Morphology.__init__(self)
        if x is None:
            theta = rand()*2 * pi
            phi = rand()*2 * pi
            x = length * sin(theta) * cos(phi)
            y = length * sin(theta) * sin(phi)
            z = length * cos(theta)
        else:
            length = (sum(array((x, y, z)) ** 2)) ** .5
        scale = arange(1, n + 1) * 1. / n
        self.x, self.y, self.z = x * scale, y * scale, z * scale
        self.length = ones(n) * length / n
        self.diameter = ones(n) * diameter
        self.area = ones(n) * pi * diameter * length / n
        self.type = type


class Soma(Morphology): # or Sphere?
    """
    A spherical soma.
    """
    def __init__(self, diameter=None):
        Morphology.__init__(self)
        self.diameter = ones(1) * diameter
        self.area = ones(1) * pi * diameter ** 2
        self.length = zeros(1)
        self.x, self.y, self.z = zeros(1), zeros(1), zeros(1)
        self.type = 'soma'

if __name__ == '__main__':
    from pylab import show
    #morpho=Morphology('mp_ma_40984_gc2.CNG.swc') # retinal ganglion cell
    morpho = Morphology('oi24rpy1.CNG.swc') # visual L3 pyramidal cell
    print len(morpho), "compartments"
    morpho.axon = None
    morpho.plot()
    #morpho=Cylinder(length=10*um,diameter=1*um,n=10)
    #morpho.plot(simple=True)
    morpho = Soma(diameter=10 * um)
    morpho.dendrite = Cylinder(length=3 * um, diameter=1 * um, n=10)
    morpho.dendrite.L = Cylinder(length=5 * um, diameter=1 * um, n=10)
    morpho.dendrite.R = Cylinder(length=7 * um, diameter=1 * um, n=10)
    morpho.dendrite.LL = Cylinder(length=3 * um, diameter=1 * um, n=10)
    morpho.axon = Morphology(n=5)
    morpho.axon.diameter = ones(5) * 1 * um
    morpho.axon.length = [1 * um, 2 * um, 1 * um, 3 * um, 1 * um]
    morpho.axon.set_coordinates()
    morpho.axon.set_area()
    morpho.plot(simple=True)
    show()



from brian import *
from time import *
#Parameters
taum = 10*msecond
taue = 5*msecond
vt_input = -54*mvolt
vt_PC= -54*mvolt
vr = -60*mvolt
E1 = -60*mvolt
start_weight=0.6
tau_pre=20*ms
tau_post=tau_pre
gmax= 1
dA_pre=.01
dA_post=-dA_pre*tau_pre/tau_post*1.05
vr_input=vr # right?

#Signal1 + singal2 transformed
eqs_source1 = '''
Y1 = .5*sin(5*t/ms)*volt + .5*sin(7*t/ms)*volt : volt
dV/dt = -V/ms : 1
'''

#Signal1 + singal2 transformed
eqs_source2 = '''
Y2 = .5*sin(5*t/ms)*volt - .5*sin(7*t/ms)*volt : volt
dV/dt = -V/ms : 1
'''

#Rule for neuron encoding a source
eqs_inputs='''
dV/dt= (-(V-E1)+ s)/taum : volt
s : volt
'''

#Rule for neuron extracting the principal component
eqs_PC1_LIF='''
dV/dt = (-(V-E1)+ge)/taum : volt
dge/dt = -ge/taue : volt
'''

#Creating neurons
source1=NeuronGroup(1, model=eqs_source1) #'neuron' representing source1
source2=NeuronGroup(1, model=eqs_source2) #'neuron' representing source2
inputs=NeuronGroup(2, model=eqs_inputs, threshold=vt_input,reset=vr_input) #2 neurons, 1 for each source
inputs.V=vr
input1 = inputs.subgroup(1) #neuron converting source1 into a spike train
input2 = inputs.subgroup(1) #neuron converting souce2 into a spike train
PC=NeuronGroup(1, model=eqs_PC1_LIF, threshold=vt_PC, reset=vr)
#neuron for extracting the first principal component
PC.V=vr

#linking analogue value of source to a LIF neuron's voltage
input1.s=linked_var(source1, 'Y1')
input2.s=linked_var(source2, 'Y2')

#Excitatory connection between the inputs and the principal component
# extracting neuron, starting weight is set in the parameter section
Cmain = Connection(inputs, PC, 'ge', weight=start_weight*volt)

#Adding STDP to connection between inputs and principal component neuron
eqs_stdp='''
dA_pre/dt = -A_pre/tau_pre:1
dA_post/dt = -A_post/tau_post:1
'''

dA_post*=gmax
dA_pre*=gmax

#post code adapted to have synaptic scaling (sum of squaes of weights = 1)
stdp_with_scaling= STDP(Cmain, eqs_stdp, pre='A_pre+=dA_pre;w+=A_post',
                        post='A_post+=dA_post;w+=A_pre; w=w/(sum(w**2)**(.5))',
                        wmax=gmax)
#stdp_w/out_scaling= STDP(Cmain, eqs_stdp, pre='A_pre+=dA_pre;w
#+=A_post',post='A_post+=dA_post;w+=A_pre', wmax=gmax)

run(100*msecond)

plot(Cmain.W.todense(), '.')

print sum(Cmain.W.todense()**2)

xlim((-.5, 1.5))
show()
